@@ -2,7 +2,7 @@
  * os_specific.h: OS specific features
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: os_specific.h,v 1.7 2002/04/28 09:05:37 sam Exp $
+ * $Id: os_specific.h,v 1.10 2002/11/11 14:39:11 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -30,7 +30,7 @@
 #   include "beos_specific.h"
 #elif defined( SYS_DARWIN )
 #   include "darwin_specific.h"
-#elif defined( WIN32 )
+#elif defined( WIN32 ) || defined( UNDER_CE )
 #   include "win32_specific.h"
 #else
 #   undef _NEED_OS_SPECIFIC_H
@@ -40,25 +40,19 @@
 extern "C" {
 #   endif
 
-/*****************************************************************************
- * main_sys_t: system specific descriptor
- ****************************************************************************/
-struct main_sys_s;
-
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
 #ifdef _NEED_OS_SPECIFIC_H
-    void system_Init ( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] );
-    void system_Configure  ( void );
-    void system_End  ( void );
+    void system_Init       ( vlc_t *, int *, char *[] );
+    void system_Configure  ( vlc_t * );
+    void system_End        ( vlc_t * );
 #else
 #   define system_Init( a, b, c ) {}
-#   define system_Configure( ) {}
-#   define system_End( ) {}
+#   define system_Configure( a ) {}
+#   define system_End( a ) {}
 #endif
 
 #   ifdef __cplusplus
 }
 #   endif
-
