@@ -2,7 +2,7 @@
  * osd.h : Constants for use with osd modules
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: osd.h,v 1.1 2003/03/23 16:38:40 sigmunau Exp $
+ * $Id: osd.h,v 1.2 2003/07/14 21:32:58 sigmunau Exp $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -25,3 +25,15 @@
 #define OSD_ALIGN_RIGHT 0x1
 #define OSD_ALIGN_TOP 0
 #define OSD_ALIGN_BOTTOM 0x2
+struct text_style_t
+{
+    int i_size;
+    uint32_t i_color;
+    vlc_bool_t b_italic;
+    vlc_bool_t b_bold;
+    vlc_bool_t b_underline;
+};
+static const text_style_t default_text_style = { 22, 0xffffff, VLC_FALSE, VLC_FALSE, VLC_FALSE };
+
+VLC_EXPORT( void, vout_ShowTextRelative, ( vout_thread_t *, char *, text_style_t *, int, int, int, mtime_t ) );
+VLC_EXPORT( void,  vout_ShowTextAbsolute, ( vout_thread_t *, char *, text_style_t *, int, int, int, mtime_t, mtime_t ) );
