@@ -2,7 +2,7 @@
  * avi.c : AVI file Stream input module for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: avi.c,v 1.42 2003/03/30 18:14:37 gbazin Exp $
+ * $Id: avi.c,v 1.53 2003/07/20 12:34:36 sigmunau Exp $
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 #include <vlc/vlc.h>
 #include <vlc/input.h>
 
-#include "video.h"
+#include "vlc_video.h"
 
 #include "libavi.h"
 
@@ -166,6 +166,9 @@ static int input_ReadInPES( input_thread_t *p_input,
     return p_pes->i_pes_size;
 }
 
+/* check if the file is interleaved */
+static vlc_bool_t AVI_Interleaved( input_thread_t *p_input );
+
 /* Test if it seems that it's a key frame */
 static int AVI_GetKeyFlag( vlc_fourcc_t i_fourcc, uint8_t *p_byte )
 {
@@ -334,7 +337,7 @@ static int AVI_PacketGetHeader( input_thread_t *p_input, avi_packet_t *p_pk )
     p_pk->i_fourcc  = GetFOURCC( p_peek );
     p_pk->i_size    = GetDWLE( p_peek + 4 );
     p_pk->i_pos     = AVI_TellAbsolute( p_input );
-    if( p_pk->i_fourcc == AVIFOURCC_LIST )
+    if( p_pk->i_fourcc == AVIFOURCC_LIST || p_pk->i_fourcc == AVIFOURCC_RIFF )
     {
         p_pk->i_type = GetFOURCC( p_peek + 8 );
     }
@@ -357,10 +360,16 @@ static int AVI_PacketNext( input_thread_t *p_input )
     {
         return VLC_EGENERIC;
     }
-    if( avi_ck.i_fourcc == AVIFOURCC_LIST && avi_ck.i_type == AVIFOURCC_rec )
+    //msg_Dbg( p_input, "skip %4.4s:%4.4s", (char*)&avi_ck.i_fourcc, (char*)&avi_ck.i_type );
+
+    if( avi_ck.i_fourcc == AVIFOURCC_LIST && ( avi_ck.i_type == AVIFOURCC_rec || avi_ck.i_type == AVIFOURCC_movi ) )
     {
         return AVI_SkipBytes( p_input, 12 );
     }
+    else if( avi_ck.i_fourcc == AVIFOURCC_RIFF && avi_ck.i_type == AVIFOURCC_AVIX )
+    {
+        return AVI_SkipBytes( p_input, 24 );
+    }
     else
     {
         return AVI_SkipBytes( p_input, __EVEN( avi_ck.i_size ) + 8 );
@@ -413,6 +422,7 @@ static int AVI_PacketSearch( input_thread_t *p_input )
         {
             case AVIFOURCC_JUNK:
             case AVIFOURCC_LIST:
+            case AVIFOURCC_RIFF:
             case AVIFOURCC_idx1:
                 return VLC_SUCCESS;
         }
@@ -472,7 +482,7 @@ static void AVI_IndexAddEntry( demux_sys_t *p_avi,
     }
 }
 
-static void AVI_IndexLoad( input_thread_t *p_input )
+static int AVI_IndexLoad_idx1( input_thread_t *p_input )
 {
     demux_sys_t *p_avi = p_input->p_demux_data;
 
@@ -493,14 +503,9 @@ static void AVI_IndexLoad( input_thread_t *p_input )
     if( !p_idx1 )
     {
         msg_Warn( p_input, "cannot find idx1 chunk, no index defined" );
-        return;
-    }
-    for( i_stream = 0; i_stream < p_avi->i_streams; i_stream++ )
-    {
-        p_avi->pp_info[i_stream]->i_idxnb  = 0;
-        p_avi->pp_info[i_stream]->i_idxmax = 0;
-        p_avi->pp_info[i_stream]->p_index  = NULL;
+        return VLC_EGENERIC;
     }
+
     /* *** calculate offset *** */
     if( p_idx1->i_entry_count > 0 &&
         p_idx1->entry[0].i_pos < p_movi->i_chunk_pos )
@@ -531,14 +536,136 @@ static void AVI_IndexLoad( input_thread_t *p_input )
             AVI_IndexAddEntry( p_avi, i_stream, &index );
         }
     }
+    return VLC_SUCCESS;
+}
+
+static void __Parse_indx( input_thread_t    *p_input,
+                          int               i_stream,
+                          avi_chunk_indx_t  *p_indx )
+{
+    demux_sys_t         *p_avi    = p_input->p_demux_data;
+    AVIIndexEntry_t     index;
+    int32_t             i;
+
+    msg_Dbg( p_input, "loading subindex(0x%x) %d entries", p_indx->i_indextype, p_indx->i_entriesinuse );
+    if( p_indx->i_indexsubtype == 0 )
+    {
+        for( i = 0; i < p_indx->i_entriesinuse; i++ )
+        {
+            index.i_id      = p_indx->i_id;
+            index.i_flags   = p_indx->idx.std[i].i_size & 0x80000000 ? 0 : AVIIF_KEYFRAME;
+            index.i_pos     = p_indx->i_baseoffset + p_indx->idx.std[i].i_offset - 8;
+            index.i_length  = p_indx->idx.std[i].i_size&0x7fffffff;
+
+            AVI_IndexAddEntry( p_avi, i_stream, &index );
+        }
+    }
+    else if( p_indx->i_indexsubtype == AVI_INDEX_2FIELD )
+    {
+        for( i = 0; i < p_indx->i_entriesinuse; i++ )
+        {
+            index.i_id      = p_indx->i_id;
+            index.i_flags   = p_indx->idx.field[i].i_size & 0x80000000 ? 0 : AVIIF_KEYFRAME;
+            index.i_pos     = p_indx->i_baseoffset + p_indx->idx.field[i].i_offset - 8;
+            index.i_length  = p_indx->idx.field[i].i_size;
+
+            AVI_IndexAddEntry( p_avi, i_stream, &index );
+        }
+    }
+    else
+    {
+        msg_Warn( p_input, "unknow subtype index(0x%x)", p_indx->i_indexsubtype );
+    }
+}
+
+static void AVI_IndexLoad_indx( input_thread_t *p_input )
+{
+    demux_sys_t         *p_avi = p_input->p_demux_data;
+    unsigned int        i_stream;
+    int32_t             i;
+
+    avi_chunk_list_t    *p_riff;
+    avi_chunk_list_t    *p_hdrl;
+
+    p_riff = (void*)AVI_ChunkFind( &p_avi->ck_root,
+                                   AVIFOURCC_RIFF, 0);
+    p_hdrl = (void*)AVI_ChunkFind( p_riff, AVIFOURCC_hdrl, 0 );
+
+    for( i_stream = 0; i_stream < p_avi->i_streams; i_stream++ )
+    {
+        avi_chunk_list_t    *p_strl;
+        avi_chunk_indx_t    *p_indx;
+
+#define p_stream  p_avi->pp_info[i_stream]
+        p_strl = (void*)AVI_ChunkFind( p_hdrl, AVIFOURCC_strl, i_stream );
+        p_indx = (void*)AVI_ChunkFind( p_strl, AVIFOURCC_indx, 0 );
+
+        if( !p_indx )
+        {
+            msg_Warn( p_input, "cannot find indx (misdetect/broken OpenDML file?)" );
+            continue;
+        }
+
+        if( p_indx->i_indextype == AVI_INDEX_OF_CHUNKS )
+        {
+            __Parse_indx( p_input, i_stream, p_indx );
+        }
+        else if( p_indx->i_indextype == AVI_INDEX_OF_INDEXES )
+        {
+            avi_chunk_indx_t    ck_sub;
+            for( i = 0; i < p_indx->i_entriesinuse; i++ )
+            {
+                if( AVI_SeekAbsolute( p_input, p_indx->idx.super[i].i_offset ) )
+                {
+                    break;
+                }
+
+                if( AVI_ChunkRead( p_input, &ck_sub, NULL, p_avi->b_seekable ) )
+                {
+                    break;
+                }
+            }
+        }
+        else
+        {
+            msg_Warn( p_input, "unknow type index(0x%x)", p_indx->i_indextype );
+        }
+#undef p_stream
+    }
+}
+
+static void AVI_IndexLoad( input_thread_t *p_input )
+{
+    demux_sys_t *p_avi = p_input->p_demux_data;
+    unsigned int i_stream;
+
+    for( i_stream = 0; i_stream < p_avi->i_streams; i_stream++ )
+    {
+        p_avi->pp_info[i_stream]->i_idxnb  = 0;
+        p_avi->pp_info[i_stream]->i_idxmax = 0;
+        p_avi->pp_info[i_stream]->p_index  = NULL;
+    }
+
+    if( p_avi->b_odml )
+    {
+        AVI_IndexLoad_indx( p_input );
+    }
+    else
+    {
+        if( AVI_IndexLoad_idx1( p_input ) )
+        {
+            /* try indx if idx1 failed as some "normal" file have indx too */
+            AVI_IndexLoad_indx( p_input );
+        }
+    }
+
     for( i_stream = 0; i_stream < p_avi->i_streams; i_stream++ )
     {
         msg_Dbg( p_input,
-                "stream[%d] creating %d index entries",
+                "stream[%d] created %d index entries",
                 i_stream,
                 p_avi->pp_info[i_stream]->i_idxnb );
     }
-
 }
 
 static void AVI_IndexCreate( input_thread_t *p_input )
@@ -596,7 +723,22 @@ static void AVI_IndexCreate( input_thread_t *p_input )
             switch( pk.i_fourcc )
             {
                 case AVIFOURCC_idx1:
+                    if( p_avi->b_odml )
+                    {
+                        avi_chunk_list_t *p_avix;
+                        p_avix = (void*)AVI_ChunkFind( &p_avi->ck_root,
+                                                       AVIFOURCC_RIFF, 1 );
+
+                        msg_Dbg( p_input, "looking for new RIFF chunk" );
+                        if( AVI_SeekAbsolute( p_input, p_avix->i_chunk_pos + 24) )
+                        {
+                            goto print_stat;
+                        }
+                        break;
+                    }
                     goto print_stat;
+                case AVIFOURCC_RIFF:
+                        msg_Dbg( p_input, "new RIFF chunk found" );
                 case AVIFOURCC_rec:
                 case AVIFOURCC_JUNK:
                     break;
@@ -609,7 +751,8 @@ static void AVI_IndexCreate( input_thread_t *p_input )
                     }
             }
         }
-        if( pk.i_pos + pk.i_size >= i_movi_end ||
+
+        if( ( !p_avi->b_odml && pk.i_pos + pk.i_size >= i_movi_end ) ||
             AVI_PacketNext( p_input ) )
         {
             break;
@@ -821,7 +964,7 @@ static int AVIInit( vlc_object_t * p_this )
     p_input->pf_demux = AVIDemux_Seekable;
     if( AVI_TestFile( p_input ) )
     {
-        msg_Warn( p_input, "avi module discarded (invalid headr)" );
+        msg_Warn( p_input, "avi module discarded (invalid header)" );
         return VLC_EGENERIC;
     }
 
@@ -844,6 +987,8 @@ static int AVIInit( vlc_object_t * p_this )
     p_avi->b_seekable = ( ( p_input->stream.b_seekable )
                         &&( p_input->stream.i_method == INPUT_METHOD_FILE ) );
     p_avi->i_movi_lastchunk_pos = 0;
+    p_avi->b_odml = VLC_FALSE;
+    p_avi->b_interleaved = VLC_FALSE;
 
     /* *** for unseekable stream, automaticaly use AVIDemux_interleaved *** */
     if( !p_avi->b_seekable || config_GetInt( p_input, "avi-interleaved" ) )
@@ -858,6 +1003,29 @@ static int AVIInit( vlc_object_t * p_this )
     }
     AVI_ChunkDumpDebug( p_input, &p_avi->ck_root );
 
+    if( AVI_ChunkCount( &p_avi->ck_root, AVIFOURCC_RIFF ) > 1 )
+    {
+        int i_count = AVI_ChunkCount( &p_avi->ck_root, AVIFOURCC_RIFF );
+        int i;
+
+        msg_Warn( p_input, "multiple riff -> OpenDML ?" );
+        for( i = 1; i < i_count; i++ )
+        {
+            avi_chunk_list_t *p_avix;
+
+            p_avix = (avi_chunk_list_t*)AVI_ChunkFind( &p_avi->ck_root,
+                                                       AVIFOURCC_RIFF, i );
+            if( p_avix->i_type == AVIFOURCC_AVIX )
+            {
+                msg_Warn( p_input, "detected OpenDML file" );
+
+                p_avi->b_odml = VLC_TRUE;
+                break;
+            }
+        }
+        p_avi->b_odml = VLC_TRUE;
+    }
+
 
     p_riff  = (avi_chunk_list_t*)AVI_ChunkFind( &p_avi->ck_root,
                                                 AVIFOURCC_RIFF, 0 );
@@ -997,20 +1165,29 @@ static int AVIInit( vlc_object_t * p_this )
                         p_avi_strf_auds->p_wf->nSamplesPerSec,
                         p_avi_strf_auds->p_wf->wBitsPerSample );
                 {
-                    char hepp[sizeof("Stream") + 10];
+                    char psz_cat[ sizeof("Stream") + 10 ];
                     input_info_category_t *p_cat;
-                    sprintf(hepp, "Stream %d", i);
-                    p_cat = input_InfoCategory( p_input, hepp);
+                    sprintf( psz_cat, _("Stream %d"), i );
+                    p_cat = input_InfoCategory( p_input, psz_cat );
                     input_AddInfo( p_cat, _("Type"), "Audio(0x%x)",
                                    p_avi_strf_auds->p_wf->wFormatTag );
                     input_AddInfo( p_cat, _("Codec"), "%4.4s",
                                    (const char*)&(p_info->i_codec) );
+                    input_AddInfo( p_cat, _("FOURCC"), "0x%x",
+                                   p_info->i_codec );
                     input_AddInfo( p_cat, _("Channels"), "%d",
                                    p_avi_strf_auds->p_wf->nChannels );
                     input_AddInfo( p_cat, _("Sample Rate"), "%d",
                                    p_avi_strf_auds->p_wf->nSamplesPerSec );
-                    input_AddInfo( p_cat, _("Bits Per Sample"), "%d",
-                                   p_avi_strf_auds->p_wf->wBitsPerSample );
+                    if( p_avi_strf_auds->p_wf->wBitsPerSample > 0
+                        && p_info->i_scale != 0 )
+                    {
+                        input_AddInfo( p_cat, _("Bits Per Sample"), "%d",
+                                       p_avi_strf_auds->p_wf->wBitsPerSample );
+                        input_AddInfo( p_cat, _("Audio Bitrate"), "%d",
+                                       p_info->i_samplesize * p_info->i_rate
+                                       / p_info->i_scale );
+                    }
                 }
                 break;
 
@@ -1033,13 +1210,15 @@ static int AVIInit( vlc_object_t * p_this )
                          (float)p_info->i_rate /
                              (float)p_info->i_scale );
                 {
-                    char hepp[sizeof("Stream") + 10];
+                    char psz_cat[ sizeof("Stream") + 10 ];
                     input_info_category_t *p_cat;
-                    sprintf(hepp, "Stream %d", i);
-                    p_cat = input_InfoCategory( p_input, hepp);
+                    sprintf( psz_cat, "Stream %d", i );
+                    p_cat = input_InfoCategory( p_input, psz_cat );
                     input_AddInfo( p_cat, _("Type"), _("Video") );
                     input_AddInfo( p_cat, _("Codec"), "%4.4s",
                                    (const char*)&(p_info->i_codec) );
+                    input_AddInfo( p_cat, _("FOURCC"), "0x%x",
+                                   p_info->i_codec );
                     input_AddInfo( p_cat, _("Resolution"), "%dx%d",
                                    p_avi_strf_vids->p_bih->biWidth,
                                    p_avi_strf_vids->p_bih->biHeight );
@@ -1076,15 +1255,17 @@ static int AVIInit( vlc_object_t * p_this )
         /* add one ES */
         vlc_mutex_lock( &p_input->stream.stream_lock );
         p_info->p_es =
-            p_es = input_AddES( p_input,
-                                p_input->stream.p_selected_program, 1+i,
-                                0 );
+            p_es = input_AddES( p_input, p_input->stream.p_selected_program,
+                                1+i, p_info->i_cat, NULL, 0 );
         vlc_mutex_unlock( &p_input->stream.stream_lock );
         p_es->i_stream_id =i; /* XXX: i don't use it */
         p_es->i_fourcc = p_info->i_fourcc;
-        p_es->i_cat = p_info->i_cat;
         if( p_es->i_cat == AUDIO_ES )
         {
+            if( i_init_size < sizeof( WAVEFORMATEX ) )
+            {
+                i_init_size = sizeof( WAVEFORMATEX );
+            }
             p_es->p_waveformatex = malloc( i_init_size );
             memcpy( p_es->p_waveformatex, p_init_data, i_init_size );
         }
@@ -1181,6 +1362,10 @@ static int AVIInit( vlc_object_t * p_this )
     {
         p_input->stream.i_mux_rate =
             p_input->stream.p_selected_area->i_size / 50 / p_avi->i_length;
+
+        msg_Dbg( p_input, "checking interleaved" );
+        p_avi->b_interleaved = AVI_Interleaved( p_input );
+        msg_Dbg( p_input, "interleaved=%s", p_avi->b_interleaved ? "yes" : "no" );
     }
     else
     {
@@ -1331,6 +1516,59 @@ static mtime_t AVI_GetPTS( avi_stream_t *p_info )
     }
 }
 
+#if 0
+static void AVI_FixPTS( avi_stream_t *p_stream, pes_packet_t *p_pes )
+{
+    data_packet_t *p_data;
+    uint8_t       *p;
+    int           i_pos = 0;
+
+    switch( p_stream->i_fourcc )
+    {
+        case VLC_FOURCC( 'm', 'p', 'g', 'a' ):
+            p_data = p_pes->p_first;
+            while( p_data )
+            {
+                p = p_data->p_payload_start;
+                while( p < p_data->p_payload_end - 2 )
+                {
+                    if( p[0] == 0xff && ( p[1]&0xe0) == 0xe0 )
+                    {
+                        mtime_t i_diff = AVI_GetDPTS( p_stream, i_pos );
+                        p_pes->i_dts += i_diff;
+                        p_pes->i_pts += i_diff;
+                        return;
+                    }
+                    p++; i_pos++;
+                }
+                p_data = p_data->p_next;
+            }
+            return;
+        case VLC_FOURCC( 'a', '5', '2', ' ' ):
+            p_data = p_pes->p_first;
+            while( p_data )
+            {
+                p = p_data->p_payload_start;
+                while( p < p_data->p_payload_end - 2 )
+                {
+                    if( p[0] == 0x0b && p[1] == 0x77 )
+                    {
+                        mtime_t i_diff = AVI_GetDPTS( p_stream, i_pos );
+                        p_pes->i_dts += i_diff;
+                        p_pes->i_pts += i_diff;
+                    }
+                    p++; i_pos++;
+                }
+                p_data = p_data->p_next;
+            }
+            return;
+        default:
+            /* we can't fix :( */
+            return;
+    }
+}
+#endif
+
 static int AVI_StreamChunkFind( input_thread_t *p_input,
                                 unsigned int i_stream )
 {
@@ -1339,7 +1577,7 @@ static int AVI_StreamChunkFind( input_thread_t *p_input,
 
     /* find first chunk of i_stream that isn't in index */
 
-    if( p_avi->i_movi_lastchunk_pos >= p_avi->i_movi_begin )
+    if( p_avi->i_movi_lastchunk_pos >= p_avi->i_movi_begin + 12 )
     {
         AVI_SeekAbsolute( p_input, p_avi->i_movi_lastchunk_pos );
         if( AVI_PacketNext( p_input ) )
@@ -1349,7 +1587,7 @@ static int AVI_StreamChunkFind( input_thread_t *p_input,
     }
     else
     {
-        AVI_SeekAbsolute( p_input, p_avi->i_movi_begin );
+        AVI_SeekAbsolute( p_input, p_avi->i_movi_begin + 12 );
     }
 
     for( ;; )
@@ -1363,17 +1601,9 @@ static int AVI_StreamChunkFind( input_thread_t *p_input,
         if( avi_pk.i_stream >= p_avi->i_streams ||
             ( avi_pk.i_cat != AUDIO_ES && avi_pk.i_cat != VIDEO_ES ) )
         {
-            switch( avi_pk.i_fourcc )
+            if( AVI_PacketNext( p_input ) )
             {
-                case AVIFOURCC_LIST:
-                    AVI_SkipBytes( p_input, 12 );
-                    break;
-                default:
-                    if( AVI_PacketNext( p_input ) )
-                    {
-                        return VLC_EGENERIC;
-                    }
-                    break;
+                return VLC_EGENERIC;
             }
         }
         else
@@ -1515,38 +1745,41 @@ static int AVI_StreamSeek( input_thread_t *p_input,
             return VLC_EGENERIC;
         }
 
-        /* search key frame */
         msg_Dbg( p_input,
                  "old:"I64Fd" %s new "I64Fd,
                  i_oldpts,
                  i_oldpts > i_date ? ">" : "<",
                  i_date );
 
-        if( i_date < i_oldpts )
+        if( p_stream->i_cat == VIDEO_ES )
         {
-            while( p_stream->i_idxposc > 0 &&
-               !( p_stream->p_index[p_stream->i_idxposc].i_flags &
-                                                            AVIIF_KEYFRAME ) )
+            /* search key frame */
+            if( i_date < i_oldpts )
             {
-                if( AVI_SetStreamChunk( p_input,
-                                        i_stream,
-                                        p_stream->i_idxposc - 1 ) )
+                while( p_stream->i_idxposc > 0 &&
+                   !( p_stream->p_index[p_stream->i_idxposc].i_flags &
+                                                                AVIIF_KEYFRAME ) )
                 {
-                    return VLC_EGENERIC;
+                    if( AVI_SetStreamChunk( p_input,
+                                            i_stream,
+                                            p_stream->i_idxposc - 1 ) )
+                    {
+                        return VLC_EGENERIC;
+                    }
                 }
             }
-        }
-        else
-        {
-            while( p_stream->i_idxposc < p_stream->i_idxnb &&
-                    !( p_stream->p_index[p_stream->i_idxposc].i_flags &
-                                                            AVIIF_KEYFRAME ) )
+            else
             {
-                if( AVI_SetStreamChunk( p_input,
-                                        i_stream,
-                                        p_stream->i_idxposc + 1 ) )
+                while( p_stream->i_idxposc < p_stream->i_idxnb &&
+                        !( p_stream->p_index[p_stream->i_idxposc].i_flags &
+                                                                AVIIF_KEYFRAME ) )
                 {
-                    return VLC_EGENERIC;
+                    if( AVI_SetStreamChunk( p_input,
+                                            i_stream,
+                                            p_stream->i_idxposc + 1 ) )
+                    {
+                        return VLC_EGENERIC;
+                    }
                 }
             }
         }
@@ -1564,6 +1797,65 @@ static int AVI_StreamSeek( input_thread_t *p_input,
 #undef p_stream
 }
 
+/*****************************************************************************
+ * AVI_Interleaved: check weither a file is interleaved or not.
+ *****************************************************************************/
+static vlc_bool_t AVI_Interleaved( input_thread_t *p_input )
+{
+    demux_sys_t     *p_sys = p_input->p_demux_data;
+    unsigned int    i;
+    mtime_t         i_time = 0;
+    vlc_bool_t      b_ret = VLC_TRUE;
+
+    int64_t         i_max;
+
+    if( p_input->stream.p_selected_area->i_size <= 100 )
+    {
+        return VLC_FALSE;
+    }
+
+    i_max = __MIN( 2000000, p_input->stream.p_selected_area->i_size / 100 );
+
+#define tk p_sys->pp_info[i]
+    while( i_time < p_sys->i_length * (mtime_t)1000000)
+    {
+        int64_t     i_ref;
+
+        i_ref = -1;
+        i_time += 50000;
+        for( i = 0; i < p_sys->i_streams; i++ )
+        {
+            while( AVI_GetPTS( tk ) < i_time && tk->i_idxposc < tk->i_idxnb - 1 )
+            {
+                tk->i_idxposc++;
+            }
+
+            if( i_ref == -1 )
+            {
+                i_ref = tk->p_index[tk->i_idxposc].i_pos;
+            }
+            if( tk->p_index[tk->i_idxposc].i_pos - i_ref > i_max ||
+                tk->p_index[tk->i_idxposc].i_pos - i_ref < -i_max ||
+                tk->p_index[tk->i_idxposc].i_length > i_max )
+            {
+                msg_Dbg( p_input, "interleaved=no because ref=%lld pos=%lld length=%d (max=%lld)",
+                         i_ref, tk->p_index[tk->i_idxposc].i_pos, tk->p_index[tk->i_idxposc].i_length, i_max  );
+                b_ret = VLC_FALSE;
+                goto exit;
+            }
+        }
+    }
+
+exit:
+    for( i = 0; i < p_sys->i_streams; i++ )
+    {
+        tk->i_idxposc = 0;
+        tk->i_idxposb = 0;
+    }
+#undef tk
+    return b_ret;
+}
+
 /*****************************************************************************
  * AVISeek: goto to i_date or i_percent
  *****************************************************************************
@@ -1582,15 +1874,18 @@ static int    AVISeek   ( input_thread_t *p_input,
 
     if( p_avi->b_seekable )
     {
-        if( !p_avi->i_length )
+        if( !p_avi->i_length || p_avi->b_interleaved )
         {
             avi_stream_t *p_stream;
-            uint64_t i_pos;
+            int64_t i_pos;
 
             /* use i_percent to create a true i_date */
-            msg_Warn( p_input,
-                      "mmh, seeking without index at %d%%"
-                      " work only for interleaved file", i_percent );
+            if( !p_avi->b_interleaved )
+            {
+                msg_Warn( p_input,
+                          "mmh, seeking without index at %d%%"
+                          " work only for interleaved file", i_percent );
+            }
 
             if( i_percent >= 100 )
             {
@@ -1837,17 +2132,8 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
     }
 #endif
 
-    /* *** send audio data to decoder if rate == DEFAULT_RATE or no video *** */
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-    if( p_input->stream.control.i_rate == DEFAULT_RATE || !b_video )
-    {
-        b_play_audio = VLC_TRUE;
-    }
-    else
-    {
-        b_play_audio = VLC_FALSE;
-    }
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
+    /* Check if we need to send the audio data to decoder */
+    b_play_audio = !p_input->stream.control.b_mute;
 
     /* init toread */
     for( i_stream = 0; i_stream < p_avi->i_streams; i_stream++ )
@@ -1936,7 +2222,7 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
         {
             /* no valid index, we will parse directly the stream
              * in case we fail we will disable all finished stream */
-            if( p_avi->i_movi_lastchunk_pos >= p_avi->i_movi_begin )
+            if( p_avi->i_movi_lastchunk_pos >= p_avi->i_movi_begin + 12 )
             {
                 AVI_SeekAbsolute( p_input, p_avi->i_movi_lastchunk_pos );
                 if( AVI_PacketNext( p_input ) )
@@ -1946,7 +2232,7 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
             }
             else
             {
-                AVI_SeekAbsolute( p_input, p_avi->i_movi_begin );
+                AVI_SeekAbsolute( p_input, p_avi->i_movi_begin + 12 );
             }
 
             for( ;; )
@@ -1962,19 +2248,11 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
                 if( avi_pk.i_stream >= p_avi->i_streams ||
                     ( avi_pk.i_cat != AUDIO_ES && avi_pk.i_cat != VIDEO_ES ) )
                 {
-                    switch( avi_pk.i_fourcc )
+                    if( AVI_PacketNext( p_input ) )
                     {
-                        case AVIFOURCC_LIST:
-                            AVI_SkipBytes( p_input, 12 );
-                            break;
-                        default:
-                            if( AVI_PacketNext( p_input ) )
-                            {
-                                msg_Warn( p_input,
-                                          "cannot skip packet, track disabled" );
-                                return( AVI_StreamStopFinishedStreams( p_input, p_avi ) ? 0 : 1 );
-                            }
-                            break;
+                        msg_Warn( p_input,
+                                  "cannot skip packet, track disabled" );
+                        return( AVI_StreamStopFinishedStreams( p_input, p_avi ) ? 0 : 1 );
                     }
                     continue;
                 }
@@ -2068,6 +2346,14 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
 
         p_pes->i_pts = AVI_GetPTS( p_stream );
 
+#if 0
+        /* fix pts for audio: ie pts sould be for the first byte of the first frame */
+        if( p_stream->i_samplesize == 1 )
+        {
+            AVI_FixPTS( p_stream, p_pes );
+        }
+#endif
+
         /* read data */
         if( p_stream->i_samplesize )
         {
@@ -2116,6 +2402,8 @@ static int AVIDemux_Seekable( input_thread_t *p_input )
                                       p_input->stream.p_selected_program,
                                       p_pes->i_pts * 9/100);
 
+            p_pes->i_rate = p_input->stream.control.i_rate;
+
             input_DecodePES( p_stream->p_es->p_decoder_fifo, p_pes );
         }
         else
@@ -2139,10 +2427,8 @@ static int AVIDemux_UnSeekable( input_thread_t *p_input )
     unsigned int i_stream;
     unsigned int i_packet;
 
-    /* *** send audio data to decoder only if rate == DEFAULT_RATE *** */
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-    b_audio = p_input->stream.control.i_rate == DEFAULT_RATE;
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
+    /* Check if we need to send the audio data to decoder */
+    b_audio = !p_input->stream.control.b_mute;
 
     input_ClockManageRef( p_input,
                           p_input->stream.p_selected_program,
@@ -2199,8 +2485,13 @@ static int AVIDemux_UnSeekable( input_thread_t *p_input )
             {
                 case AVIFOURCC_JUNK:
                 case AVIFOURCC_LIST:
+                case AVIFOURCC_RIFF:
                     return( !AVI_PacketNext( p_input ) ? 1 : 0 );
                 case AVIFOURCC_idx1:
+                    if( p_avi->b_odml )
+                    {
+                        return( !AVI_PacketNext( p_input ) ? 1 : 0 );
+                    }
                     return( 0 );    // eof
                 default:
                     msg_Warn( p_input,
@@ -2241,6 +2532,9 @@ static int AVIDemux_UnSeekable( input_thread_t *p_input )
                             input_ClockGetTS( p_input,
                                           p_input->stream.p_selected_program,
                                           AVI_GetPTS( p_stream ) * 9/100);
+
+                    p_pes->i_rate = p_input->stream.control.i_rate;
+
                     input_DecodePES( p_stream->p_es->p_decoder_fifo, p_pes );
                 }
                 else
