@@ -2,7 +2,7 @@
  * sub.c
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: sub.c,v 1.11 2003/03/30 14:24:20 gbazin Exp $
+ * $Id: sub.c,v 1.20 2003/07/27 23:58:32 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -31,8 +31,7 @@
 
 #include <vlc/vlc.h>
 #include <vlc/input.h>
-
-#include "video.h"
+#include "vlc_video.h"
 
 #include "sub.h"
 
@@ -55,28 +54,29 @@ static char *ppsz_sub_type[] = { "microdvd", "subrip", "ssa1", "ssa2-4", "vplaye
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-
+#define SUB_DELAY_LONGTEXT \
+    "Delay subtitles (in 1/10s)"
 #define SUB_FPS_LONGTEXT \
     "Override frames per second. " \
-    "It will work only with MicroDVD"
+    "It will only work with MicroDVD subtitles."
 #define SUB_TYPE_LONGTEXT \
-    "One from \"microdvd\", \"subrip\", \"ssa1\", \"ssa2-4\", \"vplayer\" \"sami\"" \
-    "(nothing for autodetection, it should always work)"
+    "One from \"microdvd\", \"subrip\", \"ssa1\", \"ssa2-4\", \"vplayer\" " \
+    "\"sami\" (nothing for autodetection, it should always work)."
 
 vlc_module_begin();
-    set_description( _("text subtitle demux") );
+    set_description( _("Text subtitles demux") );
     set_capability( "subtitle demux", 12 );
-    add_category_hint( "subtitle", NULL, VLC_TRUE );
+    add_category_hint( "Subtitles", NULL, VLC_TRUE );
         add_file( "sub-file", NULL, NULL,
-                  "subtitle file name", "subtitle file name", VLC_TRUE );
+                  "Subtitles file name", "Subtitles file name", VLC_TRUE );
         add_float( "sub-fps", 0.0, NULL,
-                   "override frames per second",
+                   "Frames per second",
                    SUB_FPS_LONGTEXT, VLC_TRUE );
         add_integer( "sub-delay", 0, NULL,
-                     "delay subtitles (in 1/10s)",
-                     "delay subtitles (in 1/10s)", VLC_TRUE );
+                     "Delay subtitles (in 1/10s)",
+                     SUB_DELAY_LONGTEXT, VLC_TRUE );
         add_string_from_list( "sub-type", NULL, ppsz_sub_type, NULL,
-                              "subtitle type",
+                              "subtitles type",
                               SUB_TYPE_LONGTEXT, VLC_TRUE );
     set_callbacks( Open, NULL );
 vlc_module_end();
@@ -93,6 +93,12 @@ static int Open ( vlc_object_t *p_this )
     p_sub->pf_seek  = sub_seek;
     p_sub->pf_close = sub_close;
 
+    /* Initialize the variables */
+    var_Create( p_this, "sub-file", VLC_VAR_FILE | VLC_VAR_DOINHERIT );
+    var_Create( p_this, "sub-fps", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+    var_Create( p_this, "sub-delay", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Create( p_this, "sub-type", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+
     return VLC_SUCCESS;
 }
 #define MAX_TRY     256
@@ -229,9 +235,9 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                        mtime_t i_microsecperframe )
 {
     text_t  txt;
+    vlc_value_t val;
 
     int     i;
-    char    *psz_file_type;
     int     i_sub_type;
     int     i_max;
     int (*pf_read_subtitle)( text_t *, subtitle_t *, mtime_t ) = NULL;
@@ -244,11 +250,13 @@ static int  sub_open ( subtitle_demux_t *p_sub,
 
     if( !psz_name || !*psz_name)
     {
-        psz_name = config_GetPsz( p_sub, "sub-file" );
-        if( !psz_name || !*psz_name )
+        var_Get( p_sub, "sub-file", &val );
+        if( !val.psz_string || !*val.psz_string )
         {
+            if( val.psz_string ) free( val.psz_string );
             return VLC_EGENERIC;
         }
+        psz_name = val.psz_string;
     }
     else
     {
@@ -265,19 +273,19 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     msg_Dbg( p_sub, "opened `%s'", psz_name );
     free( psz_name );
 
-
-    if(  config_GetFloat( p_sub, "sub-fps" ) >= 1.0 )
+    var_Get( p_sub, "sub-fps", &val );
+    if( val.i_int >= 1.0 )
     {
-        i_microsecperframe = (mtime_t)( (float)1000000 /
-                                        config_GetFloat( p_sub, "sub-fps" ) );
+        var_Get( p_sub, "sub-fps", &val );
+        i_microsecperframe = (mtime_t)( (float)1000000 / val.f_float );
     }
     else if( i_microsecperframe <= 0 )
     {
         i_microsecperframe = 40000; /* default: 25fps */
     }
 
-    psz_file_type = config_GetPsz( p_sub, "sub-type" );
-    if( psz_file_type && *psz_file_type)
+    var_Get( p_sub, "sub-type", &val);
+    if( val.psz_string && *val.psz_string )
     {
         int i;
 
@@ -289,7 +297,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                 break;
             }
             if( !strcmp( sub_read_subtitle_function[i].psz_type_name,
-                         psz_file_type ) )
+                         val.psz_string ) )
             {
                 i_sub_type = sub_read_subtitle_function[i].i_type;
                 break;
@@ -300,7 +308,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     {
         i_sub_type = SUB_TYPE_UNKNOWN;
     }
-    FREE( psz_file_type );
+    FREE( val.psz_string );
 
     /* *** Now try to autodetect subtitle format *** */
     if( i_sub_type == SUB_TYPE_UNKNOWN )
@@ -376,15 +384,14 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     {
         if( sub_read_subtitle_function[i].i_type == SUB_TYPE_UNKNOWN )
         {
-            msg_Dbg( p_input, "unknown subtitile file" );
+            msg_Dbg( p_input, "unknown subtitle file" );
             text_unload( &txt );
             return VLC_EGENERIC;
         }
 
         if( sub_read_subtitle_function[i].i_type == i_sub_type )
         {
-            msg_Dbg( p_input,
-                    "detected %s format",
+            msg_Dbg( p_input, "detected %s format",
                     sub_read_subtitle_function[i].psz_name );
             pf_read_subtitle = sub_read_subtitle_function[i].pf_read_subtitle;
             break;
@@ -425,15 +432,13 @@ static int  sub_open ( subtitle_demux_t *p_sub,
 
     /* *** add subtitle ES *** */
     vlc_mutex_lock( &p_input->stream.stream_lock );
-    p_sub->p_es = input_AddES( p_input,
-                               p_input->stream.p_selected_program,
+    p_sub->p_es = input_AddES( p_input, p_input->stream.p_selected_program,
                                0xff,    // FIXME
-                               0 );
+                               SPU_ES, NULL, 0 );
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
     p_sub->p_es->i_stream_id = 0xff;    // FIXME
     p_sub->p_es->i_fourcc    = VLC_FOURCC( 's','u','b','t' );
-    p_sub->p_es->i_cat       = SPU_ES;
 
     p_sub->i_previously_selected = 0;
     return VLC_SUCCESS;
@@ -444,7 +449,6 @@ static int  sub_open ( subtitle_demux_t *p_sub,
  *****************************************************************************/
 static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate )
 {
-
     if( p_sub->p_es->p_decoder_fifo && !p_sub->i_previously_selected )
     {
         p_sub->i_previously_selected = 1;
@@ -486,6 +490,7 @@ static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate )
             p_sub->i_subtitle++;
             continue;
         }
+        p_data->p_payload_end = p_data->p_payload_start + i_len;
 
         p_pes->i_pts =
             input_ClockGetTS( p_sub->p_input,
@@ -505,6 +510,7 @@ static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate )
         {
             p_pes->i_dts = 0;
         }
+
         p_pes->i_nb_data = 1;
         p_pes->p_first =
             p_pes->p_last = p_data;
@@ -572,6 +578,7 @@ static void  sub_fix( subtitle_demux_t *p_sub )
     mtime_t i_delay;
     int     i_index;
     int     i_done;
+    vlc_value_t val;
 
     /* *** fix order (to be sure...) *** */
     /* We suppose that there are near in order and this durty bubble sort
@@ -601,7 +608,8 @@ static void  sub_fix( subtitle_demux_t *p_sub )
     } while( !i_done );
 
     /* *** and at the end add delay *** */
-    i_delay = (mtime_t)config_GetInt( p_sub, "sub-delay" ) * 100000;
+    var_Get( p_sub, "sub-delay", &val );
+    i_delay = (mtime_t) val.i_int * 100000;
     if( i_delay != 0 )
     {
         for( i = 0; i < p_sub->i_subtitles; i++ )
