@@ -1,8 +1,8 @@
 /*****************************************************************************
  * controls.m: MacOS X interface plugin
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: controls.m,v 1.28 2003/02/12 14:22:23 hartman Exp $
+ * Copyright (C) 2002-2003 VideoLAN
+ * $Id: controls.m,v 1.47 2003/07/29 21:14:10 gbazin Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -32,49 +32,8 @@
 
 #include "intf.h"
 #include "vout.h"
-
-/*****************************************************************************
- * VLCControls interface 
- *****************************************************************************/
-@interface VLCControls : NSObject
-{
-    IBOutlet id o_open;
-    IBOutlet id o_main;
-
-    IBOutlet id o_volumeslider;
-}
-
-- (IBAction)play:(id)sender;
-- (IBAction)stop:(id)sender;
-- (IBAction)faster:(id)sender;
-- (IBAction)slower:(id)sender;
-
-- (IBAction)prev:(id)sender;
-- (IBAction)next:(id)sender;
-- (IBAction)loop:(id)sender;
-
-- (IBAction)forward:(id)sender;
-- (IBAction)backward:(id)sender;
-
-- (IBAction)volumeUp:(id)sender;
-- (IBAction)volumeDown:(id)sender;
-- (IBAction)mute:(id)sender;
-- (IBAction)volumeSliderUpdated:(id)sender;
-- (void)updateVolumeSlider;
-
-- (IBAction)halfWindow:(id)sender;
-- (IBAction)normalWindow:(id)sender;
-- (IBAction)doubleWindow:(id)sender;
-- (IBAction)fullscreen:(id)sender;
-- (IBAction)deinterlace:(id)sender;
-
-- (IBAction)toggleProgram:(id)sender;
-- (IBAction)toggleTitle:(id)sender;
-- (IBAction)toggleChapter:(id)sender;
-- (IBAction)toggleLanguage:(id)sender;
-- (IBAction)toggleVar:(id)sender;
-
-@end
+#include "open.h"
+#include "controls.h"
 
 /*****************************************************************************
  * VLCControls implementation 
@@ -107,7 +66,7 @@
         else
         {
             vlc_object_release( p_playlist );
-            [o_open openFile: nil];
+            [o_open openFileGeneric: nil];
         }
     }
 }
@@ -171,6 +130,7 @@
 
 - (IBAction)prev:(id)sender
 {
+    vlc_value_t val;
     intf_thread_t * p_intf = [NSApp getIntf];
 
     playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
@@ -190,18 +150,20 @@
     }
 
     vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
+    val.b_bool = VLC_TRUE;
 
 #define p_area p_playlist->p_input->stream.p_selected_area
-
-    if( p_area->i_part_nb > 1 && p_area->i_part > 1 )
+    if( p_area->i_part > 0 && p_area->i_part_nb > 1)
     {
-        p_area->i_part--;
-
         vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
-        input_ChangeArea( p_playlist->p_input, p_area );
+        var_Set( p_playlist->p_input, "prev-chapter", val );
+        vlc_mutex_unlock( &p_playlist->object_lock );
+    }
+    else if( p_area->i_id > 1 )
+    {
+        vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
+        var_Set( p_playlist->p_input, "prev-title", val );
         vlc_mutex_unlock( &p_playlist->object_lock );
-
-        p_intf->p_sys->b_chapter_update = VLC_TRUE;
     }
     else
     {
@@ -209,7 +171,6 @@
         vlc_mutex_unlock( &p_playlist->object_lock );
         playlist_Prev( p_playlist );
     }
-
 #undef p_area
 
     vlc_object_release( p_playlist );
@@ -217,6 +178,7 @@
 
 - (IBAction)next:(id)sender
 {
+    vlc_value_t val;
     intf_thread_t * p_intf = [NSApp getIntf];
 
     playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
@@ -225,7 +187,7 @@
     {
         return;
     }
-
+    
     vlc_mutex_lock( &p_playlist->object_lock );
 
     if( p_playlist->p_input == NULL )
@@ -236,18 +198,20 @@
     }
 
     vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
+    val.b_bool = VLC_TRUE;
 
 #define p_area p_playlist->p_input->stream.p_selected_area
-
-    if( p_area->i_part_nb > 1 && p_area->i_part + 1 < p_area->i_part_nb )
+    if( p_area->i_part < p_area->i_part_nb - 1 && p_area->i_part_nb > 1 )
     {
-        p_area->i_part++;
-
         vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
-        input_ChangeArea( p_playlist->p_input, p_area );
+        var_Set( p_playlist->p_input, "next-chapter", val );
+        vlc_mutex_unlock( &p_playlist->object_lock );
+    }
+    else if( p_area->i_id < p_playlist->p_input->stream.i_area_nb && p_playlist->p_input->stream.i_area_nb > 1 )
+    {
+        vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
+        var_Set( p_playlist->p_input, "next-title", val );
         vlc_mutex_unlock( &p_playlist->object_lock );
-
-        p_intf->p_sys->b_chapter_update = VLC_TRUE;
     }
     else
     {
@@ -255,7 +219,6 @@
         vlc_mutex_unlock( &p_playlist->object_lock );
         playlist_Next( p_playlist );
     }
-
 #undef p_area
 
     vlc_object_release( p_playlist );
@@ -289,6 +252,7 @@
         return;
     }
 
+    playlist_Play( p_playlist );
     input_Seek( p_playlist->p_input, 5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
     vlc_object_release( p_playlist );
 }
@@ -304,6 +268,7 @@
         return;
     }
 
+    playlist_Play( p_playlist );
     input_Seek( p_playlist->p_input, -5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
     vlc_object_release( p_playlist );
 }
@@ -365,297 +330,277 @@
     [o_volumeslider setFloatValue: (float)(i_volume / AOUT_VOLUME_STEP)]; 
 }
 
-- (IBAction)halfWindow:(id)sender
+- (IBAction)windowAction:(id)sender
 {
     id o_window = [NSApp keyWindow];
+    NSString *o_title = [sender title];
     NSArray *o_windows = [NSApp windows];
     NSEnumerator *o_enumerator = [o_windows objectEnumerator];
-    
-    while ((o_window = [o_enumerator nextObject]))
+    vout_thread_t   *p_vout = vlc_object_find( [NSApp getIntf], VLC_OBJECT_VOUT,
+                                              FIND_ANYWHERE );
+
+    if( p_vout != NULL )
     {
-        if( [[o_window className] isEqualToString: @"VLCWindow"] )
+        while ((o_window = [o_enumerator nextObject]))
         {
-            [o_window scaleWindowWithFactor: 0.5];
+            if( [[o_window className] isEqualToString: @"VLCWindow"] )
+            {
+                if( [o_title isEqualToString: _NS("Fullscreen") ] )
+                    [o_window toggleFullscreen];
+                else if( [o_title isEqualToString: _NS("Half Size") ] )
+                    [o_window scaleWindowWithFactor: 0.5];
+                else if( [o_title isEqualToString: _NS("Normal Size") ] )
+                    [o_window scaleWindowWithFactor: 1.0];
+                else if( [o_title isEqualToString: _NS("Double Size") ] )
+                    [o_window scaleWindowWithFactor: 2.0];
+                else if( [o_title isEqualToString: _NS("Float On Top") ] )
+                    [o_window toggleFloatOnTop];
+                else if( [o_title isEqualToString: _NS("Fit To Screen") ] )
+                {
+                    if( ![o_window isZoomed] )
+                        [o_window performZoom:self];
+                }
+            }
         }
+        vlc_object_release( (vlc_object_t *)p_vout );
     }
 }
 
-- (IBAction)normalWindow:(id)sender
+- (void)setupVarMenuItem:(NSMenuItem *)o_mi
+                    target:(vlc_object_t *)p_object
+                    var:(const char *)psz_variable
+                    selector:(SEL)pf_callback
 {
-    id o_window = [NSApp keyWindow];
-    NSArray *o_windows = [NSApp windows];
-    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
-    
-    while ((o_window = [o_enumerator nextObject]))
-    {
-        if( [[o_window className] isEqualToString: @"VLCWindow"] )
-        {
-            [o_window scaleWindowWithFactor: 1];
-        }
-    }
-}
+    vlc_value_t val, text;
+    int i_type = var_Type( p_object, psz_variable );
 
-- (IBAction)doubleWindow:(id)sender
-{
-    id o_window = [NSApp keyWindow];
-    NSArray *o_windows = [NSApp windows];
-    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
-    
-    while ((o_window = [o_enumerator nextObject]))
+    switch( i_type & VLC_VAR_TYPE )
     {
-        if( [[o_window className] isEqualToString: @"VLCWindow"] )
-        {
-            [o_window scaleWindowWithFactor: 2];
-        }
+    case VLC_VAR_VOID:
+    case VLC_VAR_BOOL:
+    case VLC_VAR_VARIABLE:
+    case VLC_VAR_STRING:
+    case VLC_VAR_INTEGER:
+        break;
+    default:
+        /* Variable doesn't exist or isn't handled */
+        return;
     }
+    
+    /* Make sure we want to display the variable */
+    if( i_type & VLC_VAR_HASCHOICE )
+    {
+        var_Change( p_object, psz_variable, VLC_VAR_CHOICESCOUNT, &val, NULL );
+        if( val.i_int == 0 ) return;
+        if( (i_type & VLC_VAR_TYPE) != VLC_VAR_VARIABLE && val.i_int == 1 )
+            return;
+    }
+    
+    /* Get the descriptive name of the variable */
+    var_Change( p_object, psz_variable, VLC_VAR_GETTEXT, &text, NULL );
+    [o_mi setTitle: [NSApp localizedString: text.psz_string ?
+                                        text.psz_string : strdup( psz_variable ) ]];
+
+    var_Get( p_object, psz_variable, &val );
+    if( i_type & VLC_VAR_HASCHOICE )
+    {
+        NSMenu *o_menu = [o_mi submenu];
+
+        [self setupVarMenu: o_menu forMenuItem: o_mi target:p_object
+                        var:psz_variable selector:pf_callback];
+        
+        if( text.psz_string ) free( text.psz_string );
+        return;
+    }
+
+    VLCMenuExt *o_data;
+    switch( i_type & VLC_VAR_TYPE )
+    {
+    case VLC_VAR_VOID:
+        o_data = [[VLCMenuExt alloc] initWithVar: psz_variable Object: p_object->i_object_id
+                Value: val ofType: i_type];
+        [o_mi setRepresentedObject: [NSValue valueWithPointer:[o_data retain]]];
+        break;
+
+    case VLC_VAR_BOOL:
+        o_data = [[VLCMenuExt alloc] initWithVar: psz_variable Object: p_object->i_object_id
+                Value: val ofType: i_type];
+        [o_mi setRepresentedObject: [NSValue valueWithPointer:[o_data retain]]];
+        [o_mi setState: val.b_bool ? TRUE : FALSE ];
+        break;
+
+    default:
+        if( text.psz_string ) free( text.psz_string );
+        return;
+    }
+
+    if( (i_type & VLC_VAR_TYPE) == VLC_VAR_STRING ) free( val.psz_string );
+    if( text.psz_string ) free( text.psz_string );
 }
 
 
-- (IBAction)fullscreen:(id)sender
+- (void)setupVarMenu:(NSMenu *)o_menu
+                    forMenuItem: (NSMenuItem *)o_parent
+                    target:(vlc_object_t *)p_object
+                    var:(const char *)psz_variable
+                    selector:(SEL)pf_callback
 {
-    id o_window = [NSApp keyWindow];
-    NSArray *o_windows = [NSApp windows];
-    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
-    
-    while ((o_window = [o_enumerator nextObject]))
-    {
-        if( [[o_window className] isEqualToString: @"VLCWindow"] )
-        {
-            [o_window toggleFullscreen];
-        }
-    }
-}
+    vlc_value_t val, val_list, text_list;
+    int i_type, i, i_nb_items;
 
-- (IBAction)deinterlace:(id)sender
-{
-    intf_thread_t * p_intf = [NSApp getIntf];
-    BOOL bEnable = [sender state] == NSOffState;
-    
-    if( bEnable )
+    /* remove previous items */
+    i_nb_items = [o_menu numberOfItems];
+    for( i = 0; i < i_nb_items; i++ )
     {
-        config_PutPsz( p_intf, "filter", "deinterlace" );
-	config_PutPsz( p_intf, "deinterlace-mode",
-                    [[sender title] lossyCString] );
+        [o_menu removeItemAtIndex: 0];
+    }
+
+    /* Check the type of the object variable */
+    i_type = var_Type( p_object, psz_variable );
+
+    /* Make sure we want to display the variable */
+    if( i_type & VLC_VAR_HASCHOICE )
+    {
+        var_Change( p_object, psz_variable, VLC_VAR_CHOICESCOUNT, &val, NULL );
+        if( val.i_int == 0 ) return;
+        if( (i_type & VLC_VAR_TYPE) != VLC_VAR_VARIABLE && val.i_int == 1 )
+            return;
     }
     else
     {
-        config_PutPsz( p_intf, "filter", NULL );
+        return;
     }
-}
 
-- (IBAction)toggleProgram:(id)sender
-{
-    NSMenuItem * o_mi = (NSMenuItem *)sender;
-    intf_thread_t * p_intf = [NSApp getIntf];
+    switch( i_type & VLC_VAR_TYPE )
+    {
+    case VLC_VAR_VOID:
+    case VLC_VAR_BOOL:
+    case VLC_VAR_VARIABLE:
+    case VLC_VAR_STRING:
+    case VLC_VAR_INTEGER:
+        break;
+    default:
+        /* Variable doesn't exist or isn't handled */
+        return;
+    }
 
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-    if( p_playlist == NULL )
+    if( var_Get( p_object, psz_variable, &val ) < 0 )
     {
         return;
     }
 
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input == NULL )
+    if( var_Change( p_object, psz_variable, VLC_VAR_GETLIST,
+                    &val_list, &text_list ) < 0 )
     {
-        vlc_mutex_unlock( &p_playlist->object_lock );
-        vlc_object_release( p_playlist );
+        if( (i_type & VLC_VAR_TYPE) == VLC_VAR_STRING ) free( val.psz_string );
         return;
     }
 
-    if( [o_mi state] == NSOffState )
-    {
-        u16 i_program_id = [o_mi tag];
+    /* make (un)sensitive */
+    [o_parent setEnabled: ( val_list.p_list->i_count > 1 )];
 
-        input_ChangeProgram( p_playlist->p_input, i_program_id );
-        input_SetStatus( p_playlist->p_input, INPUT_STATUS_PLAY );
+    for( i = 0; i < val_list.p_list->i_count; i++ )
+    {
+        vlc_value_t another_val;
+        NSMenuItem * o_lmi;
+        NSString *o_title = @"";
+        VLCMenuExt *o_data;
+
+        switch( i_type & VLC_VAR_TYPE )
+        {
+        case VLC_VAR_VARIABLE:
+
+            /* This is causing crashes for the moment.
+            o_title = [NSApp localizedString: text_list.p_list->p_values[i].psz_string ?
+                text_list.p_list->p_values[i].psz_string : val_list.p_list->p_values[i].psz_string ];
+            
+            o_data = [[VLCMenuExt alloc] initWithVar: strdup(psz_variable) Object: p_object->i_object_id
+                Value: val ofType: i_type];
+            [o_lmi setRepresentedObject: [NSValue valueWithPointer:[o_data retain]]];
+
+            // Create a submenu
+            NSMenu *o_menu = [o_lmi submenu];
+
+            [self setupVarMenu: o_menu forMenuItem: o_lmi target:p_object
+                            var:psz_variable selector:pf_callback];
+*/
+            return;
+
+        case VLC_VAR_STRING:
+            another_val.psz_string =
+                strdup(val_list.p_list->p_values[i].psz_string);
+
+            o_title = [NSApp localizedString: text_list.p_list->p_values[i].psz_string ?
+                text_list.p_list->p_values[i].psz_string : val_list.p_list->p_values[i].psz_string ];
+
+            o_lmi = [o_menu addItemWithTitle: o_title action: pf_callback keyEquivalent: @""];
+            o_data = [[VLCMenuExt alloc] initWithVar: strdup(psz_variable) Object: p_object->i_object_id
+                    Value: another_val ofType: i_type];
+            [o_lmi setRepresentedObject: [NSValue valueWithPointer:[o_data retain]]];
+            [o_lmi setTarget: self];
+            
+            if( !strcmp( val.psz_string, val_list.p_list->p_values[i].psz_string ) )
+                [o_lmi setState: TRUE ];
+
+            break;
+
+        case VLC_VAR_INTEGER:
+
+             o_title = text_list.p_list->p_values[i].psz_string ?
+                                 [NSApp localizedString: strdup( text_list.p_list->p_values[i].psz_string )] :
+                                 [NSString stringWithFormat: @"%d",
+                                 val_list.p_list->p_values[i].i_int];
+
+            o_lmi = [[o_menu addItemWithTitle: o_title action: pf_callback keyEquivalent: @""] retain ];
+            o_data = [[VLCMenuExt alloc] initWithVar: strdup(psz_variable) Object: p_object->i_object_id
+                    Value: val_list.p_list->p_values[i] ofType: i_type];
+            [o_lmi setRepresentedObject: [NSValue valueWithPointer:[ o_data retain]]];
+            [o_lmi setTarget: self];
+
+            if( val_list.p_list->p_values[i].i_int == val.i_int )
+                [o_lmi setState: TRUE ];
+            break;
+
+        default:
+          break;
+        }
     }
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
-}
-
-- (IBAction)toggleTitle:(id)sender
-{
-    NSMenuItem * o_mi = (NSMenuItem *)sender;
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-    if( p_playlist == NULL )
-    {
-        return;
-    }
-
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input == NULL )
-    {
-        vlc_mutex_unlock( &p_playlist->object_lock );
-        vlc_object_release( p_playlist );
-        return;
-    }
-
-    if( [o_mi state] == NSOffState )
-    {
-        int i_title = [o_mi tag];
-
-#define p_input p_playlist->p_input
-        input_ChangeArea( p_input, p_input->stream.pp_areas[i_title] );
-        input_SetStatus( p_input, INPUT_STATUS_PLAY );
-#undef p_input
-    }
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
-}
-
-- (IBAction)toggleChapter:(id)sender
-{
-    NSMenuItem * o_mi = (NSMenuItem *)sender;
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-    if( p_playlist == NULL )
-    {
-        return;
-    }
-
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input == NULL )
-    {
-        vlc_mutex_unlock( &p_playlist->object_lock );
-        vlc_object_release( p_playlist );
-        return;
-    }
-
-    if( [o_mi state] == NSOffState )
-    {
-        int i_chapter = [o_mi tag];
-
-#define p_input p_playlist->p_input
-        p_input->stream.p_selected_area->i_part = i_chapter;
-        input_ChangeArea( p_input, p_input->stream.p_selected_area );
-        input_SetStatus( p_input, INPUT_STATUS_PLAY );
-#undef p_input
-    }
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
-}
-
-- (IBAction)toggleLanguage:(id)sender
-{
-    NSMenuItem * o_mi = (NSMenuItem *)sender;
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-    if( p_playlist == NULL )
-    {
-        return;
-    }
-
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input == NULL )
-    {
-        vlc_mutex_unlock( &p_playlist->object_lock );
-        vlc_object_release( p_playlist );
-        return;
-    }
-
-#if 0
-    /* We do not use this code, because you need to start stop .avi for
-     * it to work, so not very useful now  --hartman */
-    if ( [o_mi state] == NSOffState && [o_mi tag] == 2000 )
-    {
-        NSOpenPanel *o_open_panel = [NSOpenPanel openPanel];
-        
-        [o_open_panel setAllowsMultipleSelection: NO];
-        [o_open_panel setTitle: _NS("Open subtitle file")];
-        [o_open_panel setPrompt: _NS("Open")];
     
-        if( [o_open_panel runModalForDirectory: nil 
-                file: nil types: nil] == NSOKButton )
-        {
-            NSString *o_filename = [[o_open_panel filenames] objectAtIndex: 0];
-            config_PutPsz( p_intf, "sub-file", strdup( [o_filename cString] ));
-        }
-    }
-#endif
-
-#define p_input p_playlist->p_input
-
-    if( !p_intf->p_sys->b_audio_update )
-    {
-        NSValue * o_value = [o_mi representedObject];
-        es_descriptor_t * p_es = [o_value pointerValue];
-
-        if( [o_mi state] == NSOnState )
-        {
-            /* we just have one ES to disable */
-            input_ToggleES( p_input, p_es, 0 );
-        }
-        else
-        {
-            unsigned int i;
-            int i_cat = [o_mi tag];
-
-            vlc_mutex_lock( &p_input->stream.stream_lock );
-
-#define ES p_input->stream.pp_selected_es[i]
-
-            /* unselect the selected ES in the same class */
-            for( i = 0; i < p_input->stream.i_selected_es_number; i++ )
-            {
-                if( ES->i_cat == i_cat )
-                {
-                    vlc_mutex_unlock( &p_input->stream.stream_lock );
-                    input_ToggleES( p_input, ES, 0 );
-                    vlc_mutex_lock( &p_input->stream.stream_lock );
-                    break;
-                }
-            }
-
-#undef ES
-
-            vlc_mutex_unlock( &p_input->stream.stream_lock );
-
-            input_ToggleES( p_input, p_es, 1 );
-        }
-    }
-
-#undef p_input
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
+    /* clean up everything */
+    if( (i_type & VLC_VAR_TYPE) == VLC_VAR_STRING ) free( val.psz_string );
+    var_Change( p_object, psz_variable, VLC_VAR_FREELIST, &val_list, &text_list );
 }
 
 - (IBAction)toggleVar:(id)sender
 {
-    NSMenuItem * o_mi = (NSMenuItem *)sender;
-    
-    if( [o_mi state] == NSOffState )
+    NSMenuItem *o_mi = (NSMenuItem *)sender;
+    VLCMenuExt *o_data = [[o_mi representedObject] pointerValue];
+    [NSThread detachNewThreadSelector: @selector(toggleVarThread:)
+        toTarget: self withObject: o_data];
+
+    return;
+}
+
+- (int)toggleVarThread: (id)_o_data
+{
+    vlc_object_t *p_object;
+    NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
+    VLCMenuExt *o_data = (VLCMenuExt *)_o_data;
+
+    vlc_thread_set_priority( [NSApp getIntf] , VLC_THREAD_PRIORITY_LOW );
+
+    p_object = (vlc_object_t *)vlc_object_get( [NSApp getIntf],
+                                    [o_data objectID] );
+
+    if( p_object != NULL )
     {
-        const char * psz_variable = (const char *)[o_mi tag];
-        char * psz_value = [NSApp delocalizeString: [o_mi title]];
-        vlc_object_t * p_object = (vlc_object_t *)
-            [[o_mi representedObject] pointerValue];
-        vlc_value_t val;
-        /* psz_string sucks */
-        val.psz_string = (char *)psz_value;
-
-        if ( var_Set( p_object, psz_variable, val ) < 0 )
-        {
-            msg_Warn( p_object, "cannot set variable (%s)", psz_value );
-        }
-
-        free( psz_value );
+        var_Set( p_object, strdup([o_data name]), [o_data value] );
+        vlc_object_release( p_object );
+        [o_pool release];
+        return VLC_TRUE;
     }
+    [o_pool release];
+    return VLC_EGENERIC;
 }
 
 @end
@@ -665,7 +610,6 @@
 - (BOOL)validateMenuItem:(NSMenuItem *)o_mi
 {
     BOOL bEnabled = TRUE;
-    NSMenu * o_menu = [o_mi menu];
     intf_thread_t * p_intf = [NSApp getIntf];
 
     playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
@@ -713,7 +657,7 @@
             if( p_input != NULL )
             {
                 vlc_mutex_lock( &p_input->stream.stream_lock );
-                bEnabled |= p_input->stream.p_selected_area->i_part_nb > 1;
+                bEnabled |= p_input->stream.i_area_nb > 1;
                 vlc_mutex_unlock( &p_input->stream.stream_lock );
             }
         }
@@ -746,50 +690,35 @@
     else if( [[o_mi title] isEqualToString: _NS("Fullscreen")] ||
                 [[o_mi title] isEqualToString: _NS("Half Size")] ||
                 [[o_mi title] isEqualToString: _NS("Normal Size")] ||
-                [[o_mi title] isEqualToString: _NS("Double Size")])    
+                [[o_mi title] isEqualToString: _NS("Double Size")] ||
+                [[o_mi title] isEqualToString: _NS("Fit To Screen")] ||
+                [[o_mi title] isEqualToString: _NS("Float On Top")] )
     {
         id o_window;
         NSArray *o_windows = [NSApp windows];
         NSEnumerator *o_enumerator = [o_windows objectEnumerator];
         bEnabled = FALSE;
         
-        while ((o_window = [o_enumerator nextObject]))
+        if ( [[o_mi title] isEqualToString: _NS("Float On Top")] )
         {
-            if( [[o_window className] isEqualToString: @"VLCWindow"] )
-            {
-                bEnabled = TRUE;
-                break;
-            }
+            int i_state = config_GetInt( p_playlist, "macosx-float" ) ?
+                      NSOnState : NSOffState;
+            [o_mi setState: i_state];
         }
-    }
-    else if( o_menu != nil && 
-             [[o_menu title] isEqualToString: _NS("Deinterlace")] )
-    {
-        char * psz_filter = config_GetPsz( p_intf, "filter" );
         
-        if( psz_filter != NULL )
+        vout_thread_t   *p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                              FIND_ANYWHERE );
+        if( p_vout != NULL )
         {
-            free( psz_filter );
-            
-            psz_filter = config_GetPsz( p_intf, "deinterlace-mode" );
-        }
-
-        if( psz_filter != NULL )
-        {
-            if( strcmp( psz_filter, [[o_mi title] lossyCString] ) == 0 )
+            while ((o_window = [o_enumerator nextObject]))
             {
-                [o_mi setState: NSOnState];
+                if( [[o_window className] isEqualToString: @"VLCWindow"] )
+                {
+                    bEnabled = TRUE;
+                    break;
+                }
             }
-            else
-            {
-                [o_mi setState: NSOffState];
-            }
-
-            free( psz_filter );
-        }
-        else
-        {
-            [o_mi setState: NSOffState];
+            vlc_object_release( (vlc_object_t *)p_vout );
         }
     }
 
@@ -803,3 +732,54 @@
 }
 
 @end
+
+/*****************************************************************************
+ * VLCMenuExt implementation 
+ *****************************************************************************
+ * Object connected to a playlistitem which remembers the data belonging to
+ * the variable of the autogenerated menu
+ *****************************************************************************/
+@implementation VLCMenuExt
+
+- (id)initWithVar: (const char *)_psz_name Object: (int)i_id
+        Value: (vlc_value_t)val ofType: (int)_i_type
+{
+    self = [super init];
+
+    if( self != nil )
+    {
+        psz_name = strdup( _psz_name );
+        i_object_id = i_id;
+        value = val;
+        i_type = _i_type;
+    }
+
+    return( self );
+}
+
+- (void)dealloc
+{
+    free( psz_name );
+}
+
+- (char *)name
+{
+    return psz_name;
+}
+
+- (int)objectID
+{
+    return i_object_id;
+}
+
+- (vlc_value_t)value
+{
+    return value;
+}
+
+- (int)type
+{
+    return i_type;
+}
+
+@end
