@@ -2,7 +2,7 @@
  * intf.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: intf.m,v 1.90 2003/06/17 14:43:22 hartman Exp $
+ * $Id: intf.m,v 1.93 2003/07/27 23:05:41 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -465,15 +465,9 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
 - (BOOL)application:(NSApplication *)o_app openFile:(NSString *)o_filename
 {
-    intf_thread_t * p_intf = [NSApp getIntf];
-    
-    config_PutPsz( p_intf, "sub-file", "" );
-    config_PutInt( p_intf, "sub-delay", 0 );
-    config_PutFloat( p_intf, "sub-fps", 0.0 );
-    config_PutPsz( p_intf, "sout", "" );
-    
+    NSDictionary *o_dic = [NSDictionary dictionaryWithObjectsAndKeys: o_filename, @"ITEM_URL", nil];
     [o_playlist appendArray:
-        [NSArray arrayWithObject: o_filename] atPos: -1 enqueue: NO];
+        [NSArray arrayWithObject: o_dic] atPos: -1 enqueue: NO];
             
     return( TRUE );
 }
@@ -669,8 +663,8 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
             /* control buttons for free pace streams */
             b_control = p_input->stream.b_pace_control; 
 
-            /* chapters */
-            b_chapters = p_input->stream.p_selected_area->i_part_nb > 1; 
+            /* chapters & titles */
+            b_chapters = p_input->stream.i_area_nb > 1; 
 
             /* play status */
             p_intf->p_sys->b_play_status = 
@@ -733,7 +727,10 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
                 /* release the lock to be able to seek */
                 vlc_mutex_unlock( &p_input->stream.stream_lock );
+                vlc_mutex_unlock( &p_playlist->object_lock );
+                playlist_Play( p_playlist );
                 input_Seek( p_input, i_seek, INPUT_SEEK_SET );
+                vlc_mutex_lock( &p_playlist->object_lock );
                 vlc_mutex_lock( &p_input->stream.stream_lock );
 
                 /* update the old value */
