@@ -1,8 +1,8 @@
 /*****************************************************************************
  * playlist.h: MacOS X interface plugin
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: playlist.h,v 1.5 2003/02/13 14:16:41 hartman Exp $
+ * Copyright (C) 2002-2003 VideoLAN
+ * $Id: playlist.h,v 1.10 2003/07/27 23:05:41 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <thedj@users.sourceforge.net>
@@ -27,7 +27,7 @@
  *****************************************************************************/
 @interface VLCPlaylistView : NSTableView
 {
-
+    NSColor *o_striped_row_color;
 }
 
 @end
@@ -37,6 +37,8 @@
  *****************************************************************************/
 @interface VLCPlaylist : NSObject
 {
+    int i_moveRow;
+        
     IBOutlet id o_table_view;
 
     IBOutlet id o_ctx_menu;
@@ -55,8 +57,9 @@
 - (IBAction)deleteItems:(id)sender;
 - (IBAction)selectAll:(id)sender;
 
-- (void)appendArray:(NSArray*)o_array atPos:(int)i_pos enqueue:(BOOL)b_enqueue;
+- (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue;
+
+- (void)updateRowSelection;
 - (void)playlistUpdated;
-- (void)updateState;
 
 @end
