@@ -2,7 +2,7 @@
  * vout.m: MacOS X video output plugin
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: vout.m,v 1.43 2003/03/19 13:55:43 hartman Exp $
+ * $Id: vout.m,v 1.50 2003/06/03 23:17:43 massiot Exp $
  *
  * Authors: Colin Delacroix <colin@zoy.org>
  *          Florian G. Pflug <fgp@phlo.org>
@@ -148,24 +148,21 @@ int E_(OpenVideo) ( vlc_object_t *p_this )
         return( 1 );
     } 
 
-    if( vout_ChromaCmp( p_vout->render.i_chroma, VLC_FOURCC('I','4','2','0') ) )
+    /* Damn QT isn't thread safe. so keep a lock in the p_vlc object */
+    vlc_mutex_lock( &p_vout->p_vlc->quicktime_lock );
+
+    err = FindCodec( kYUV420CodecType, bestSpeedCodec,
+                        nil, &p_vout->p_sys->img_dc );
+    
+    vlc_mutex_unlock( &p_vout->p_vlc->quicktime_lock );
+    if( err == noErr && p_vout->p_sys->img_dc != 0 )
     {
-        err = FindCodec( kYUV420CodecType, bestSpeedCodec,
-                         nil, &p_vout->p_sys->img_dc );
-        if( err == noErr && p_vout->p_sys->img_dc != 0 )
-        {
-            p_vout->output.i_chroma = VLC_FOURCC('I','4','2','0');
-            p_vout->p_sys->i_codec = kYUV420CodecType;
-        }
-        else
-        {
-            msg_Err( p_vout, "failed to find an appropriate codec" );
-        }
+        p_vout->output.i_chroma = VLC_FOURCC('I','4','2','0');
+        p_vout->p_sys->i_codec = kYUV420CodecType;
     }
     else
     {
-        msg_Err( p_vout, "chroma 0x%08x not supported",
-                         p_vout->render.i_chroma );
+        msg_Err( p_vout, "failed to find an appropriate codec" );
     }
 
     if( p_vout->p_sys->img_dc == 0 )
@@ -173,7 +170,7 @@ int E_(OpenVideo) ( vlc_object_t *p_this )
         free( p_vout->p_sys->p_matrix );
         DisposeHandle( (Handle)p_vout->p_sys->h_img_descr );
         free( p_vout->p_sys );
-        return( 1 );        
+        return VLC_EGENERIC;        
     }
 
     NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
@@ -181,14 +178,16 @@ int E_(OpenVideo) ( vlc_object_t *p_this )
     if( [o_screens count] > 0 && var_Type( p_vout, "video-device" ) == 0 )
     {
         int i = 1;
-        vlc_value_t val;
+        vlc_value_t val, text;
         NSScreen * o_screen;
 
         int i_option = config_GetInt( p_vout, "macosx-vdev" );
 
-        var_Create( p_vout, "video-device", VLC_VAR_STRING |
+        var_Create( p_vout, "video-device", VLC_VAR_INTEGER |
                                             VLC_VAR_HASCHOICE ); 
-
+        text.psz_string = _("Video device");
+        var_Change( p_vout, "video-device", VLC_VAR_SETTEXT, &text, NULL );
+        
         NSEnumerator * o_enumerator = [o_screens objectEnumerator];
 
         while( (o_screen = [o_enumerator nextObject]) != NULL )
@@ -200,14 +199,15 @@ int E_(OpenVideo) ( vlc_object_t *p_this )
                       "%s %d (%dx%d)", _("Screen"), i,
                       (int)s_rect.size.width, (int)s_rect.size.height ); 
 
-            val.psz_string = psz_temp;
-            var_Change( p_vout, "video-device", VLC_VAR_ADDCHOICE, &val );
+            text.psz_string = psz_temp;
+            val.i_int = i;
+            var_Change( p_vout, "video-device",
+                        VLC_VAR_ADDCHOICE, &val, &text );
 
             if( ( i - 1 ) == i_option )
             {
                 var_Set( p_vout, "video-device", val );
             }
-
             i++;
         }
 
@@ -818,6 +818,20 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
     }
 }
 
+- (void)toggleFloatOnTop
+{
+    if( config_GetInt( p_vout, "macosx-float" ) )
+    {
+        config_PutInt( p_vout, "macosx-float", 0 );
+        [p_vout->p_sys->o_window setLevel: NSNormalWindowLevel];
+    }
+    else
+    {
+        config_PutInt( p_vout, "macosx-float", 1 );
+        [p_vout->p_sys->o_window setLevel: NSStatusWindowLevel];
+    }
+}
+
 - (void)toggleFullscreen
 {
     p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
@@ -836,6 +850,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 - (void)keyDown:(NSEvent *)o_event
 {
     unichar key = 0;
+    vlc_value_t val;
 
     if( [[o_event characters] length] )
     {
@@ -863,6 +878,33 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
             input_SetStatus( p_vout, INPUT_STATUS_PAUSE );
             break;
 
+        case (unichar)0xf700: /* arrow up */
+            val.psz_string = "UP";
+            var_Set( p_vout, "key-pressed", val );
+            break;
+
+        case (unichar)0xf701: /* arrow down */
+            val.psz_string = "DOWN";
+            var_Set( p_vout, "key-pressed", val );
+            break;
+
+        case (unichar)0xf702: /* arrow left */
+            val.psz_string = "LEFT";
+            var_Set( p_vout, "key-pressed", val );
+            break;
+
+        case (unichar)0xf703: /* arrow right */
+            val.psz_string = "RIGHT";
+            var_Set( p_vout, "key-pressed", val );
+            break;
+
+        case (unichar)0xd: /* return */
+        case (unichar)0x3: /* enter */
+            val.psz_string = "ENTER";
+            var_Set( p_vout, "key-pressed", val );
+            break;
+
+
         default:
             [super keyDown: o_event];
             break;
@@ -1200,11 +1242,10 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
     }
     else
     {
-        unsigned int i_index = 0;
         NSArray *o_screens = [NSScreen screens];
-
-        if( !sscanf( val.psz_string, _("Screen %d"), &i_index ) ||
-            [o_screens count] < i_index )
+        unsigned int i_index = val.i_int;
+        
+        if( [o_screens count] < i_index )
         {
             o_screen = [NSScreen mainScreen];
             b_main_screen = 1;
@@ -1215,9 +1256,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
             o_screen = [o_screens objectAtIndex: i_index];
             config_PutInt( p_vout, "macosx-vdev", i_index );
             b_main_screen = (i_index == 0);
-        } 
-
-        free( val.psz_string );
+        }
     } 
 
     if( p_vout->b_fullscreen )
@@ -1256,6 +1295,13 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
             backing: NSBackingStoreBuffered
             defer: NO screen: o_screen];
 
+        [p_vout->p_sys->o_window setAlphaValue: config_GetFloat( p_vout, "macosx-opaqueness" )];
+        
+        if( config_GetInt( p_vout, "macosx-float" ) )
+        {
+            [p_vout->p_sys->o_window setLevel: NSStatusWindowLevel];
+        }
+        
         if( !p_vout->p_sys->b_pos_saved )   
         {
             [p_vout->p_sys->o_window center];
