@@ -2,7 +2,7 @@
  * copy.c
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: copy.c,v 1.4 2003/01/23 15:52:04 sam Exp $
+ * $Id: copy.c,v 1.11 2003/07/20 23:30:07 gbazin Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -44,10 +44,10 @@ typedef struct packetizer_thread_s
     decoder_fifo_t          *p_fifo;
 
     /* Output properties */
-    sout_input_t            *p_sout_input;
-    sout_packet_format_t    output_format;
+    sout_packetizer_input_t *p_sout_input;
+    sout_format_t           output_format;
 
-    mtime_t i_pts_start;
+//    mtime_t i_last_pts;
 
 } packetizer_thread_t;
 
@@ -144,6 +144,7 @@ static int InitThread( packetizer_thread_t *p_pack )
 
     switch( p_pack->p_fifo->i_fourcc )
     {
+        /* video */
         case VLC_FOURCC( 'm', '4', 's', '2'):
         case VLC_FOURCC( 'M', '4', 'S', '2'):
         case VLC_FOURCC( 'm', 'p', '4', 's'):
@@ -168,10 +169,6 @@ static int InitThread( packetizer_thread_t *p_pack )
             p_pack->output_format.i_fourcc = VLC_FOURCC( 'm', 'p', 'g', 'v' );
             p_pack->output_format.i_cat = VIDEO_ES;
             break;
-        case VLC_FOURCC( 'm', 'p', 'g', 'a' ):
-            p_pack->output_format.i_fourcc = VLC_FOURCC( 'm', 'p', 'g', 'a' );
-            p_pack->output_format.i_cat = AUDIO_ES;
-            break;
 
         case VLC_FOURCC( 'd', 'i', 'v', '1' ):
         case VLC_FOURCC( 'D', 'I', 'V', '1' ):
@@ -243,11 +240,160 @@ static int InitThread( packetizer_thread_t *p_pack )
             p_pack->output_format.i_fourcc = VLC_FOURCC( 'd', 'v', 's', 'l' );
             p_pack->output_format.i_cat = VIDEO_ES;
             break;
+        case VLC_FOURCC( 'S', 'V', 'Q', '1' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'S', 'V', 'Q', '1' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'S', 'V', 'Q', '3' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'S', 'V', 'Q', '3' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+
+        case VLC_FOURCC( 'I', '4', '2', '0' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'I', '4', '2', '0' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'I', '4', '2', '2' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'I', '4', '2', '2' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'R', 'V', '1', '5' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'R', 'V', '1', '5' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'R', 'V', '1', '6' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'R', 'V', '1', '6' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'R', 'V', '2', '4' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'R', 'V', '2', '4' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'R', 'V', '3', '2' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'R', 'V', '3', '2' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+        case VLC_FOURCC( 'G', 'R', 'E', 'Y' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'G', 'R', 'E', 'Y' );
+            p_pack->output_format.i_cat = VIDEO_ES;
+            break;
+
+        /* audio */
+        case VLC_FOURCC( 'm', 'p', 'g', 'a' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'm', 'p', 'g', 'a' );
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        case VLC_FOURCC( 'w', 'm', 'a', '1' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'w', 'm', 'a', '1' );
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        case VLC_FOURCC( 'w', 'm', 'a', '2' ):
+            p_pack->output_format.i_fourcc = VLC_FOURCC( 'w', 'm', 'a', '2' );
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        case VLC_FOURCC( 'a', 'r', 'a', 'w' ):
+        {
+            WAVEFORMATEX *p_wf = (WAVEFORMATEX*)p_pack->p_fifo->p_waveformatex;
+            if( p_wf )
+            {
+                switch( ( p_wf->wBitsPerSample + 7 ) / 8 )
+                {
+                    case 1:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('u','8',' ',' ');
+                        break;
+                    case 2:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','1','6','l');
+                        break;
+                    case 3:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','2','4','l');
+                        break;
+                    case 4:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','3','2','l');
+                        break;
+                    default:
+                        msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                        return VLC_EGENERIC;
+                }
+            }
+            else
+            {
+                msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                return VLC_EGENERIC;
+            }
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        }
+        case VLC_FOURCC( 't', 'w', 'o', 's' ):
+        {
+            WAVEFORMATEX *p_wf = (WAVEFORMATEX*)p_pack->p_fifo->p_waveformatex;
+            if( p_wf )
+            {
+                switch( ( p_wf->wBitsPerSample + 7 ) / 8 )
+                {
+                    case 1:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','8',' ',' ');
+                        break;
+                    case 2:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','1','6','b');
+                        break;
+                    case 3:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','2','4','b');
+                        break;
+                    case 4:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','3','2','b');
+                        break;
+                    default:
+                        msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                        return VLC_EGENERIC;
+                }
+            }
+            else
+            {
+                msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                return VLC_EGENERIC;
+            }
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        }
+        case VLC_FOURCC( 's', 'o', 'w', 't' ):
+        {
+            WAVEFORMATEX *p_wf = (WAVEFORMATEX*)p_pack->p_fifo->p_waveformatex;
+            if( p_wf )
+            {
+                switch( ( p_wf->wBitsPerSample + 7 ) / 8 )
+                {
+                    case 1:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','8',' ',' ');
+                        break;
+                    case 2:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','1','6','l');
+                        break;
+                    case 3:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','2','4','l');
+                        break;
+                    case 4:
+                        p_pack->output_format.i_fourcc = VLC_FOURCC('s','3','2','l');
+                        break;
+                    default:
+                        msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                        return VLC_EGENERIC;
+                }
+            }
+            else
+            {
+                msg_Err( p_pack->p_fifo, "unknown raw audio sample size !!" );
+                return VLC_EGENERIC;
+            }
+            p_pack->output_format.i_cat = AUDIO_ES;
+            break;
+        }
 
         default:
-            p_pack->output_format.i_fourcc = p_pack->p_fifo->i_fourcc;
-            p_pack->output_format.i_cat = UNKNOWN_ES;
-            break;
+            msg_Err( p_pack->p_fifo, "unknown es type !!" );
+            return VLC_EGENERIC;
+            //p_pack->output_format.i_fourcc = p_pack->p_fifo->i_fourcc;
+            //p_pack->output_format.i_cat = UNKNOWN_ES;
+            //break;
     }
 
     switch( p_pack->output_format.i_cat )
@@ -257,12 +403,32 @@ static int InitThread( packetizer_thread_t *p_pack )
                 WAVEFORMATEX *p_wf = (WAVEFORMATEX*)p_pack->p_fifo->p_waveformatex;
                 if( p_wf )
                 {
-                    p_pack->output_format.p_format = malloc( sizeof( WAVEFORMATEX ) + p_wf->cbSize );
-                    memcpy( p_pack->output_format.p_format, p_wf, sizeof( WAVEFORMATEX ) + p_wf->cbSize );
+                    p_pack->output_format.i_sample_rate = p_wf->nSamplesPerSec;
+                    p_pack->output_format.i_channels    = p_wf->nChannels;
+                    p_pack->output_format.i_block_align = p_wf->nBlockAlign;
+                    p_pack->output_format.i_bitrate     = p_wf->nAvgBytesPerSec * 8;
+                    p_pack->output_format.i_extra_data  = p_wf->cbSize;
+                    if( p_wf->cbSize  > 0 )
+                    {
+                        p_pack->output_format.p_extra_data =
+                            malloc( p_pack->output_format.i_extra_data );
+                        memcpy( p_pack->output_format.p_extra_data,
+                                &p_wf[1],
+                                p_pack->output_format.i_extra_data );
+                    }
+                    else
+                    {
+                        p_pack->output_format.p_extra_data = NULL;
+                    }
                 }
                 else
                 {
-                    p_pack->output_format.p_format = NULL;
+                    p_pack->output_format.i_sample_rate = 0;
+                    p_pack->output_format.i_channels    = 0;
+                    p_pack->output_format.i_block_align = 0;
+                    p_pack->output_format.i_bitrate     = 0;
+                    p_pack->output_format.i_extra_data  = 0;
+                    p_pack->output_format.p_extra_data  = NULL;
                 }
             }
             break;
@@ -270,29 +436,34 @@ static int InitThread( packetizer_thread_t *p_pack )
         case VIDEO_ES:
             {
                 BITMAPINFOHEADER *p_bih = (BITMAPINFOHEADER*)p_pack->p_fifo->p_bitmapinfoheader;
+
+                p_pack->output_format.i_bitrate = 0;
                 if( p_bih )
                 {
-                    p_pack->output_format.p_format = malloc( p_bih->biSize );
-                    memcpy( p_pack->output_format.p_format, p_bih, p_bih->biSize );
-                    if( p_pack->output_format.i_fourcc == VLC_FOURCC( 'm', 'p', '4', 'v' ) )
+                    p_pack->output_format.i_width  = p_bih->biWidth;
+                    p_pack->output_format.i_height = p_bih->biHeight;
+                    p_pack->output_format.i_extra_data  = p_bih->biSize - sizeof( BITMAPINFOHEADER );
+                    if( p_pack->output_format.i_extra_data > 0 )
                     {
-                        p_bih->biCompression = VLC_FOURCC( 'd', 'i', 'v', 'x' );
+                        p_pack->output_format.p_extra_data =
+                            malloc( p_pack->output_format.i_extra_data );
+                        memcpy( p_pack->output_format.p_extra_data,
+                                &p_bih[1],
+                                p_pack->output_format.i_extra_data );
                     }
-                    else
-                    {
-                        p_bih->biCompression = p_pack->output_format.i_fourcc;
-                    }
-
                 }
                 else
                 {
-                    p_pack->output_format.p_format = NULL;
+                    p_pack->output_format.i_width  = 0;
+                    p_pack->output_format.i_height = 0;
+                    p_pack->output_format.i_extra_data  = 0;
+                    p_pack->output_format.p_extra_data  = NULL;
                 }
             }
             break;
+
         default:
-            p_pack->output_format.p_format = NULL;
-            break;
+            return VLC_EGENERIC;
     }
 
     p_pack->p_sout_input =
@@ -302,10 +473,10 @@ static int InitThread( packetizer_thread_t *p_pack )
     if( !p_pack->p_sout_input )
     {
         msg_Err( p_pack->p_fifo, "cannot add a new stream" );
-        return( -1 );
+        return VLC_EGENERIC;
     }
-    p_pack->i_pts_start = -1;
-    return( 0 );
+//    p_pack->i_last_pts = 0;
+    return( VLC_SUCCESS );
 }
 
 /*****************************************************************************
@@ -315,7 +486,8 @@ static void PacketizeThread( packetizer_thread_t *p_pack )
 {
     sout_buffer_t   *p_sout_buffer;
     pes_packet_t    *p_pes;
-    ssize_t          i_size;
+    ssize_t         i_size;
+    mtime_t         i_pts;
 
     /* **** get samples count **** */
     input_ExtractPES( p_pack->p_fifo, &p_pes );
@@ -324,11 +496,17 @@ static void PacketizeThread( packetizer_thread_t *p_pack )
         p_pack->p_fifo->b_error = 1;
         return;
     }
-    if( p_pack->i_pts_start < 0 && p_pes->i_pts > 0 )
+    i_pts = p_pes->i_pts;
+
+    if( i_pts <= 0 ) //&& p_pack->i_last_pts <= 0 )
     {
-        p_pack->i_pts_start = p_pes->i_pts;
+        msg_Dbg( p_pack->p_fifo, "need pts != 0" );
+        input_DeletePES( p_pack->p_fifo->p_packets_mgt, p_pes );
+        return;
     }
+
     i_size = p_pes->i_pes_size;
+
 //    msg_Dbg( p_pack->p_fifo, "pes size:%d", i_size );
     if( i_size > 0 )
     {
@@ -336,11 +514,12 @@ static void PacketizeThread( packetizer_thread_t *p_pack )
         data_packet_t   *p_data;
         ssize_t          i_buffer;
 
-        p_sout_buffer = 
+        p_sout_buffer =
             sout_BufferNew( p_pack->p_sout_input->p_sout, i_size );
         if( !p_sout_buffer )
         {
             p_pack->p_fifo->b_error = 1;
+            input_DeletePES( p_pack->p_fifo->p_packets_mgt, p_pes );
             return;
         }
         /* TODO: memcpy of the pes packet */
@@ -350,7 +529,7 @@ static void PacketizeThread( packetizer_thread_t *p_pack )
         {
             ssize_t i_copy;
 
-            i_copy = __MIN( p_data->p_payload_end - p_data->p_payload_start, 
+            i_copy = __MIN( p_data->p_payload_end - p_data->p_payload_start,
                             i_size - i_buffer );
             if( i_copy > 0 )
             {
@@ -361,14 +540,14 @@ static void PacketizeThread( packetizer_thread_t *p_pack )
             i_buffer += i_copy;
         }
         p_sout_buffer->i_length = 0;
-        p_sout_buffer->i_dts = p_pes->i_pts - p_pack->i_pts_start;
-        p_sout_buffer->i_pts = p_pes->i_pts - p_pack->i_pts_start;
+        p_sout_buffer->i_dts = i_pts; //p_pes->i_pts - p_pack->i_pts_start;
+        p_sout_buffer->i_pts = i_pts; //p_pes->i_pts - p_pack->i_pts_start;
         p_sout_buffer->i_bitrate = 0;
 
         input_ShowPES( p_pack->p_fifo, &p_pes_next );
         if( p_pes_next )
         {
-            p_sout_buffer->i_length = p_pes_next->i_pts - p_pes->i_pts;
+            p_sout_buffer->i_length = p_pes_next->i_pts - i_pts;
         }
         sout_InputSendBuffer( p_pack->p_sout_input,
                                p_sout_buffer );
@@ -387,6 +566,7 @@ static void EndThread ( packetizer_thread_t *p_pack)
     {
         sout_InputDelete( p_pack->p_sout_input );
     }
+    free( p_pack );
 }
 
 static void input_ShowPES( decoder_fifo_t *p_fifo, pes_packet_t **pp_pes )
