@@ -2,7 +2,7 @@
  * vout.c: Windows DirectX video output display method
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: directx.c,v 1.16 2003/03/30 18:14:39 gbazin Exp $
+ * $Id: directx.c,v 1.21 2003/05/25 11:31:54 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -45,8 +45,6 @@
 #include <windows.h>
 #include <ddraw.h>
 
-#include "netutils.h"
-
 #include "vout.h"
 
 /*****************************************************************************
@@ -86,22 +84,26 @@ static void DirectXGetDDrawCaps   ( vout_thread_t *p_vout );
 static int  DirectXLockSurface    ( vout_thread_t *p_vout, picture_t *p_pic );
 static int  DirectXUnlockSurface  ( vout_thread_t *p_vout, picture_t *p_pic );
 
+/* Object variables callbacks */
+static int OnTopCallback( vlc_object_t *, char const *,
+                          vlc_value_t, vlc_value_t, void * );
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define ON_TOP_TEXT N_("always on top")
-#define ON_TOP_LONGTEXT N_("place the directx window on top of other windows")
-#define HW_YUV_TEXT N_("use hardware YUV->RGB conversions")
+#define ON_TOP_TEXT N_("Always on top")
+#define ON_TOP_LONGTEXT N_("Place the directx window on top of other windows")
+#define HW_YUV_TEXT N_("Use hardware YUV->RGB conversions")
 #define HW_YUV_LONGTEXT N_( \
     "Try to use hardware acceleration for YUV->RGB conversions. " \
     "This option doesn't have any effect when using overlays." )
-#define SYSMEM_TEXT N_("use video buffers in system memory")
+#define SYSMEM_TEXT N_("Use video buffers in system memory")
 #define SYSMEM_LONGTEXT N_( \
     "Create video buffers in system memory instead of video memory. This " \
     "isn't recommended as usually using video memory allows to benefit from " \
     "more hardware acceleration (like rescaling or YUV->RGB conversions). " \
     "This option doesn't have any effect when using overlays." )
-#define TRIPLEBUF_TEXT N_("use triple buffering for overlays")
+#define TRIPLEBUF_TEXT N_("Use triple buffering for overlays")
 #define TRIPLEBUF_LONGTEXT N_( \
     "Try to use triple bufferring when using YUV overlays. That results in " \
     "much better video quality (no flickering)." )
@@ -134,7 +136,7 @@ vlc_module_end();
 static int OpenVideo( vlc_object_t *p_this )
 {
     vout_thread_t * p_vout = (vout_thread_t *)p_this;
-    vlc_value_t val;
+    vlc_value_t val, text;
 
     /* Allocate structure */
     p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
@@ -220,8 +222,12 @@ static int OpenVideo( vlc_object_t *p_this )
 
     /* Add a variable to indicate if the window should be on top of others */
     var_Create( p_vout, "directx-on-top", VLC_VAR_BOOL );
+    text.psz_string = _("Always on top");
+    var_Change( p_vout, "directx-on-top", VLC_VAR_SETTEXT, &text, NULL );
     val.b_bool = config_GetInt( p_vout, "directx-on-top" );
     var_Set( p_vout, "directx-on-top", val );
+    p_vout->p_sys->b_on_top_change = val.b_bool; 
+    var_AddCallback( p_vout, "directx-on-top", OnTopCallback, NULL );
 
     return VLC_SUCCESS;
 
@@ -377,9 +383,6 @@ static void CloseVideo( vlc_object_t *p_this )
 static int Manage( vout_thread_t *p_vout )
 {
     WINDOWPLACEMENT window_placement;
-    HWND hwnd;
-    HMENU hMenu;
-    vlc_value_t val;
 
     /* If we do not control our window, we check for geometry changes
      * ourselves because the parent might not send us its events. */
@@ -429,6 +432,8 @@ static int Manage( vout_thread_t *p_vout )
     if( p_vout->i_changes & VOUT_FULLSCREEN_CHANGE
         || p_vout->p_sys->i_changes & VOUT_FULLSCREEN_CHANGE )
     {
+        vlc_value_t val;
+
         p_vout->b_fullscreen = ! p_vout->b_fullscreen;
 
         /* We need to switch between Maximized and Normal sized window */
@@ -453,6 +458,10 @@ static int Manage( vout_thread_t *p_vout )
 
         SetWindowPlacement( p_vout->p_sys->hwnd, &window_placement );
 
+        /* Update the object variable and trigger callback */
+        val.b_bool = p_vout->b_fullscreen;
+        var_Set( p_vout, "fullscreen", val );
+
         p_vout->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
         p_vout->p_sys->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
     }
@@ -474,30 +483,34 @@ static int Manage( vout_thread_t *p_vout )
     /*
      * "Always on top" status change
      */
-    hwnd = p_vout->p_sys->hwnd;
-    hMenu = GetSystemMenu( hwnd , FALSE );
-    var_Get( p_vout, "directx-on-top", &val );
-    if( val.b_bool )
+    if( p_vout->p_sys->b_on_top_change )
     {
+        vlc_value_t val;
+        HMENU hMenu = GetSystemMenu( p_vout->p_sys->hwnd, FALSE );
+
+        var_Get( p_vout, "directx-on-top", &val );
+ 
         /* Set the window on top if necessary */
-        if( !( GetWindowLong( hwnd, GWL_EXSTYLE ) & WS_EX_TOPMOST ) )
+        if( val.b_bool && !( GetWindowLong( p_vout->p_sys->hwnd, GWL_EXSTYLE )
+                           & WS_EX_TOPMOST ) )
         {
             CheckMenuItem( hMenu, IDM_TOGGLE_ON_TOP,
                            MF_BYCOMMAND | MFS_CHECKED );
-            SetWindowPos( hwnd, HWND_TOPMOST, 0, 0, 0, 0,
-                           SWP_NOSIZE | SWP_NOMOVE );
+            SetWindowPos( p_vout->p_sys->hwnd, HWND_TOPMOST, 0, 0, 0, 0,
+                          SWP_NOSIZE | SWP_NOMOVE );
         }
-    }
-    else
-    {
+        else
         /* The window shouldn't be on top */
-        if( GetWindowLong( hwnd, GWL_EXSTYLE ) & WS_EX_TOPMOST )
+        if( !val.b_bool && ( GetWindowLong( p_vout->p_sys->hwnd, GWL_EXSTYLE )
+                           & WS_EX_TOPMOST ) )
         {
             CheckMenuItem( hMenu, IDM_TOGGLE_ON_TOP,
                            MF_BYCOMMAND | MFS_UNCHECKED );
-            SetWindowPos( hwnd, HWND_NOTOPMOST, 0, 0, 0, 0,
+            SetWindowPos( p_vout->p_sys->hwnd, HWND_NOTOPMOST, 0, 0, 0, 0,
                           SWP_NOSIZE | SWP_NOMOVE );
         }
+
+        p_vout->p_sys->b_on_top_change = VLC_FALSE;
     }
 
     /* Check if the event thread is still running */
@@ -552,7 +565,7 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
                                             DDBLT_ASYNC, &ddbltfx );
         if( dxresult != DD_OK )
         {
-            msg_Warn( p_vout, "could not blit surface (error %i)", dxresult );
+            msg_Warn( p_vout, "could not blit surface (error %li)", dxresult );
             return;
         }
 
@@ -569,7 +582,7 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
                                              NULL, DDFLIP_WAIT );
         if( dxresult != DD_OK )
         {
-            msg_Warn( p_vout, "could not flip overlay (error %i)", dxresult );
+            msg_Warn( p_vout, "could not flip overlay (error %li)", dxresult );
         }
 
         /* set currently displayed pic */
@@ -690,7 +703,7 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
                                            &p_display, NULL );
     if( dxresult != DD_OK )
     {
-        msg_Err( p_vout, "cannot get primary surface (error %i)", dxresult );
+        msg_Err( p_vout, "cannot get primary surface (error %li)", dxresult );
         return VLC_EGENERIC;
     }
 
@@ -702,7 +715,7 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
     if ( dxresult != DD_OK )
     {
         msg_Err( p_vout, "cannot query IDirectDrawSurface2 interface "
-                         "(error %i)", dxresult );
+                         "(error %li)", dxresult );
         return VLC_EGENERIC;
     }
 
@@ -719,7 +732,7 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
     if( dxresult != DD_OK )
     {
         msg_Warn( p_vout, "DirectXUpdateOverlay GetPixelFormat failed "
-                          "(error %i)", dxresult );
+                          "(error %li)", dxresult );
     }
     p_vout->p_sys->i_colorkey = (DWORD)((( p_vout->p_sys->i_rgb_colorkey
                                            * pixel_format.dwRBitMask) / 255)
@@ -749,7 +762,7 @@ static int DirectXCreateClipper( vout_thread_t *p_vout )
                                            &p_vout->p_sys->p_clipper, NULL );
     if( dxresult != DD_OK )
     {
-        msg_Warn( p_vout, "cannot create clipper (error %i)", dxresult );
+        msg_Warn( p_vout, "cannot create clipper (error %li)", dxresult );
         goto error;
     }
 
@@ -758,7 +771,7 @@ static int DirectXCreateClipper( vout_thread_t *p_vout )
                                           p_vout->p_sys->hwnd);
     if( dxresult != DD_OK )
     {
-        msg_Warn( p_vout, "cannot attach clipper to window (error %i)",
+        msg_Warn( p_vout, "cannot attach clipper to window (error %li)",
                           dxresult );
         goto error;
     }
@@ -768,7 +781,7 @@ static int DirectXCreateClipper( vout_thread_t *p_vout )
                                              p_vout->p_sys->p_clipper);
     if( dxresult != DD_OK )
     {
-        msg_Warn( p_vout, "cannot attach clipper to surface (error %i)",
+        msg_Warn( p_vout, "cannot attach clipper to surface (error %li)",
                           dxresult );
         goto error;
     }
@@ -891,7 +904,7 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
     if ( dxresult != DD_OK )
     {
         msg_Err( p_vout, "cannot query IDirectDrawSurface2 interface "
-                         "(error %i)", dxresult );
+                         "(error %li)", dxresult );
         *pp_surface_final = NULL;
         return VLC_EGENERIC;
     }
@@ -1109,10 +1122,38 @@ static int NewPictureVec( vout_thread_t *p_vout, picture_t *p_pic,
     {
         if( p_vout->p_sys->b_hw_yuv )
         {
-            i_ret = DirectXCreateSurface( p_vout, &p_surface,
-                                          p_vout->output.i_chroma,
-                                          0 /* no overlay */,
-                                          0 /* no back buffers */ );
+            DWORD i_codes;
+            DWORD *pi_codes;
+            vlc_bool_t b_result = VLC_FALSE;
+
+            /* Check if the chroma is supported first. This is required
+             * because a few buggy drivers don't mind creating the surface
+             * even if they don't know about the chroma. */
+            if( IDirectDraw2_GetFourCCCodes( p_vout->p_sys->p_ddobject,
+                                             &i_codes, NULL ) )
+            {
+                pi_codes = malloc( i_codes * sizeof(DWORD) );
+                if( pi_codes && IDirectDraw2_GetFourCCCodes(
+                    p_vout->p_sys->p_ddobject, &i_codes, pi_codes ) )
+                {
+                    for( i = 0; i < (int)i_codes; i++ )
+                    {
+                        if( p_vout->output.i_chroma == pi_codes[i] )
+                        {
+                            b_result = VLC_TRUE;
+                            break;
+                        }
+                    }
+                }
+            }
+
+            if( b_result )
+                i_ret = DirectXCreateSurface( p_vout, &p_surface,
+                                              p_vout->output.i_chroma,
+                                              0 /* no overlay */,
+                                              0 /* no back buffers */ );
+            else
+                p_vout->p_sys->b_hw_yuv = VLC_FALSE;
         }
 
         if( i_ret || !p_vout->p_sys->b_hw_yuv )
@@ -1382,7 +1423,7 @@ static void DirectXGetDDrawCaps( vout_thread_t *p_vout )
     else
     {
         BOOL bHasOverlay, bHasOverlayFourCC, bCanClipOverlay,
-             bHasColorKey, bCanStretch;
+             bHasColorKey, bCanStretch, bCanBltFourcc;
 
         /* Determine if the hardware supports overlay surfaces */
         bHasOverlay = ((ddcaps.dwCaps & DDCAPS_OVERLAY) ==
@@ -1399,15 +1440,22 @@ static void DirectXGetDDrawCaps( vout_thread_t *p_vout )
         /* Determine if the hardware supports scaling of the overlay surface */
         bCanStretch = ((ddcaps.dwCaps & DDCAPS_OVERLAYSTRETCH) ==
                        DDCAPS_OVERLAYSTRETCH) ? TRUE : FALSE;
+        /* Determine if the hardware supports color conversion during a blit */
+        bCanBltFourcc = ((ddcaps.dwCaps & DDCAPS_BLTFOURCC ) ==
+                        DDCAPS_BLTFOURCC) ? TRUE : FALSE;
+
         msg_Dbg( p_vout, "DirectDraw Capabilities: overlay=%i yuvoverlay=%i "
-                         "can_clip_overlay=%i colorkey=%i stretch=%i",
+                         "can_clip_overlay=%i colorkey=%i stretch=%i "
+                         "bltfourcc=%i",
                          bHasOverlay, bHasOverlayFourCC, bCanClipOverlay,
-                         bHasColorKey, bCanStretch );
+                         bHasColorKey, bCanStretch, bCanBltFourcc );
 
         /* Overlay clipping support is interesting for us as it means we can
          * get rid of the colorkey alltogether */
         p_vout->p_sys->b_caps_overlay_clipping = bCanClipOverlay;
 
+        /* Don't ask for troubles */
+        if( !bCanBltFourcc ) p_vout->p_sys->b_hw_yuv = FALSE; 
     }
 }
 
@@ -1485,3 +1533,15 @@ static int DirectXUnlockSurface( vout_thread_t *p_vout, picture_t *p_pic )
     else
         return VLC_EGENERIC;
 }
+
+/*****************************************************************************
+ * object variables callbacks: a bunch of object variables are used by the
+ * interfaces to interact with the vout.
+ *****************************************************************************/
+static int OnTopCallback( vlc_object_t *p_this, char const *psz_cmd,
+                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    p_vout->p_sys->b_on_top_change = VLC_TRUE;
+    return VLC_SUCCESS;
+}
