@@ -5,7 +5,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: video_output.c,v 1.212 2003/02/09 23:42:06 sigmunau Exp $
+ * $Id: video_output.c,v 1.231 2003/07/29 22:20:53 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -35,8 +35,9 @@
 #   include <sys/times.h>
 #endif
 
-#include "video.h"
+#include "vlc_video.h"
 #include "video_output.h"
+#include <vlc/input.h>                 /* for input_thread_t and i_pts_delay */
 
 #if defined( SYS_DARWIN )
 #include "darwin_specific.h"
@@ -56,6 +57,12 @@ static int      BinaryLog         ( uint32_t );
 static void     MaskToShift       ( int *, int *, uint32_t );
 static void     InitWindowSize    ( vout_thread_t *, int *, int * );
 
+/* Object variables callbacks */
+static int FullscreenCallback( vlc_object_t *, char const *,
+                               vlc_value_t, vlc_value_t, void * );
+static int DeinterlaceCallback( vlc_object_t *, char const *,
+                                vlc_value_t, vlc_value_t, void * );
+
 /*****************************************************************************
  * vout_Request: find a video output thread, create one, or destroy one.
  *****************************************************************************
@@ -68,25 +75,27 @@ vout_thread_t * __vout_Request ( vlc_object_t *p_this, vout_thread_t *p_vout,
 {
     if( !i_width || !i_height || !i_chroma )
     {
-        /* Reattach video output to p_vlc before bailing out */
+        /* Reattach video output to input before bailing out */
         if( p_vout )
         {
-            char *psz_sout = config_GetPsz( p_this, "sout" );
+            vlc_object_t *p_input;
 
-            if( !psz_sout || !*psz_sout )
+            p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_PARENT );
+
+            if( p_input )
             {
                 vlc_object_detach( p_vout );
-                vlc_object_attach( p_vout, p_this->p_vlc );
+                vlc_object_attach( p_vout, p_input );
+
+                vlc_object_release( p_input );
             }
             else
             {
                 vlc_object_detach( p_vout );
-                vlc_object_release( p_vout );
+                /* vlc_object_release( p_vout ); */
                 vout_Destroy( p_vout );
             }
-            if( psz_sout ) free( psz_sout );
         }
-
         return NULL;
     }
 
@@ -101,7 +110,21 @@ vout_thread_t * __vout_Request ( vlc_object_t *p_this, vout_thread_t *p_vout,
 
         if( !p_vout )
         {
-            p_vout = vlc_object_find( p_this, VLC_OBJECT_VOUT, FIND_ANYWHERE );
+            vlc_object_t *p_input;
+
+            p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_PARENT );
+            if( p_input )
+            {
+                p_vout = vlc_object_find( p_input, VLC_OBJECT_VOUT,
+                                          FIND_CHILD );
+                /* only first children of p_input for unused vout */
+                if( p_vout && p_vout->p_parent != p_input )
+                {
+                    vlc_object_release( p_vout );
+                    p_vout = NULL;
+                }
+                vlc_object_release( p_input );
+            }
         }
     }
 
@@ -179,10 +202,11 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
                                unsigned int i_width, unsigned int i_height,
                                vlc_fourcc_t i_chroma, unsigned int i_aspect )
 {
-    vout_thread_t * p_vout;                             /* thread descriptor */
-    int             i_index;                                /* loop variable */
-    char          * psz_plugin;
-    vlc_value_t     val;
+    vout_thread_t  * p_vout;                            /* thread descriptor */
+    input_thread_t * p_input_thread;
+    int              i_index;                               /* loop variable */
+    char           * psz_plugin;
+    vlc_value_t      val, text;
 
     /* Allocate descriptor */
     p_vout = vlc_object_create( p_parent, VLC_OBJECT_VOUT );
@@ -192,88 +216,12 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
         return NULL;
     }
 
-    var_Create( p_vout, "intf-change", VLC_VAR_BOOL );
-    val.b_bool = VLC_TRUE;
-    var_Set( p_vout, "intf-change", val );
-
-    p_vout->b_override_aspect = VLC_FALSE;
-
-    /* If the parent is not a VOUT object, that means we are at the start of
-     * the video output pipe */
-    if( p_parent->i_object_type != VLC_OBJECT_VOUT )
-    {
-        char *psz_aspect = config_GetPsz( p_parent, "aspect-ratio" );
-
-        /* Check whether the user tried to override aspect ratio */
-        if( psz_aspect )
-        {
-            unsigned int i_new_aspect = i_aspect;
-            char *psz_parser = strchr( psz_aspect, ':' );
-
-            if( psz_parser )
-            {
-                *psz_parser++ = '\0';
-                i_new_aspect = atoi( psz_aspect ) * VOUT_ASPECT_FACTOR
-                                                  / atoi( psz_parser );
-            }
-            else
-            {
-                i_new_aspect = i_width * VOUT_ASPECT_FACTOR
-                                       * atof( psz_aspect )
-                                       / i_height;
-            }
-
-            free( psz_aspect );
-
-            if( i_new_aspect && i_new_aspect != i_aspect )
-            {
-                int i_pgcd = ReduceHeight( i_new_aspect );
-
-                msg_Dbg( p_vout, "overriding source aspect ratio to %i:%i",
-                         i_new_aspect / i_pgcd, VOUT_ASPECT_FACTOR / i_pgcd );
-
-                i_aspect = i_new_aspect;
-
-                p_vout->b_override_aspect = VLC_TRUE;
-            }
-        }
-
-        /* Look for the default filter configuration */
-        p_vout->psz_filter_chain = config_GetPsz( p_parent, "filter" );
-    }
-    else
-    {
-        /* continue the parent's filter chain */
-        char *psz_end;
-
-        psz_end = strchr( ((vout_thread_t *)p_parent)->psz_filter_chain, ':' );
-        if( psz_end && *(psz_end+1) )
-            p_vout->psz_filter_chain = strdup( psz_end+1 );
-        else p_vout->psz_filter_chain = NULL;
-    }
-
-    /* Choose the video output module */
-    if( !p_vout->psz_filter_chain || !*p_vout->psz_filter_chain )
-    {
-        psz_plugin = config_GetPsz( p_parent, "vout" );
-    }
-    else
-    {
-        /* the filter chain is a string list of filters separated by double
-         * colons */
-        char *psz_end;
-
-        psz_end = strchr( p_vout->psz_filter_chain, ':' );
-        if( psz_end )
-            psz_plugin = strndup( p_vout->psz_filter_chain,
-                                  psz_end - p_vout->psz_filter_chain );
-        else psz_plugin = strdup( p_vout->psz_filter_chain );
-    }
-
     /* Initialize pictures and subpictures - translation tables and functions
      * will be initialized later in InitThread */
     for( i_index = 0; i_index < 2 * VOUT_MAX_PICTURES; i_index++)
     {
+        p_vout->p_picture[i_index].pf_lock = NULL;
+        p_vout->p_picture[i_index].pf_unlock = NULL;
         p_vout->p_picture[i_index].i_status = FREE_PICTURE;
         p_vout->p_picture[i_index].i_type   = EMPTY_PICTURE;
     }
@@ -332,17 +280,107 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     var_Create( p_vout, "mouse-clicked", VLC_VAR_INTEGER );
     var_Create( p_vout, "key-pressed", VLC_VAR_STRING );
 
-    /* user requested fullscreen? */
-    if( config_GetInt( p_vout, "fullscreen" ) )
+    var_Create( p_vout, "intf-change", VLC_VAR_BOOL );
+    val.b_bool = VLC_TRUE;
+    var_Set( p_vout, "intf-change", val );
+
+    /* Initialize locks */
+    vlc_mutex_init( p_vout, &p_vout->picture_lock );
+    vlc_mutex_init( p_vout, &p_vout->subpicture_lock );
+    vlc_mutex_init( p_vout, &p_vout->change_lock );
+
+    /* Attach the new object now so we can use var inheritance below */
+    vlc_object_attach( p_vout, p_parent );
+
+    /* Create a few object variables we'll need later on */
+    var_Create( p_vout, "aspect-ratio", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "width", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "height", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "zoom", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+
+    p_vout->b_override_aspect = VLC_FALSE;
+
+    /* If the parent is not a VOUT object, that means we are at the start of
+     * the video output pipe */
+    if( p_parent->i_object_type != VLC_OBJECT_VOUT )
     {
-        p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
+        var_Get( p_vout, "aspect-ratio", &val );
+
+        /* Check whether the user tried to override aspect ratio */
+        if( val.psz_string )
+        {
+            unsigned int i_new_aspect = i_aspect;
+            char *psz_parser = strchr( val.psz_string, ':' );
+
+            if( psz_parser )
+            {
+                *psz_parser++ = '\0';
+                i_new_aspect = atoi( val.psz_string ) * VOUT_ASPECT_FACTOR
+                                                      / atoi( psz_parser );
+            }
+            else
+            {
+                i_new_aspect = i_width * VOUT_ASPECT_FACTOR
+                                       * atof( val.psz_string )
+                                       / i_height;
+            }
+
+            free( val.psz_string );
+
+            if( i_new_aspect && i_new_aspect != i_aspect )
+            {
+                int i_pgcd = ReduceHeight( i_new_aspect );
+
+                msg_Dbg( p_vout, "overriding source aspect ratio to %i:%i",
+                         i_new_aspect / i_pgcd, VOUT_ASPECT_FACTOR / i_pgcd );
+
+                p_vout->render.i_aspect = i_new_aspect;
+
+                p_vout->b_override_aspect = VLC_TRUE;
+            }
+        }
+
+        /* Look for the default filter configuration */
+        var_Create( p_vout, "filter", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+        var_Get( p_vout, "filter", &val );
+        p_vout->psz_filter_chain = val.psz_string;
+    }
+    else
+    {
+        /* continue the parent's filter chain */
+        char *psz_end;
+
+        psz_end = strchr( ((vout_thread_t *)p_parent)->psz_filter_chain, ':' );
+        if( psz_end && *(psz_end+1) )
+            p_vout->psz_filter_chain = strdup( psz_end+1 );
+        else p_vout->psz_filter_chain = NULL;
+    }
+
+    /* Choose the video output module */
+    if( !p_vout->psz_filter_chain || !*p_vout->psz_filter_chain )
+    {
+        var_Create( p_vout, "vout", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+        var_Get( p_vout, "vout", &val );
+        psz_plugin = val.psz_string;
+    }
+    else
+    {
+        /* the filter chain is a string list of filters separated by double
+         * colons */
+        char *psz_end;
+
+        psz_end = strchr( p_vout->psz_filter_chain, ':' );
+        if( psz_end )
+            psz_plugin = strndup( p_vout->psz_filter_chain,
+                                  psz_end - p_vout->psz_filter_chain );
+        else psz_plugin = strdup( p_vout->psz_filter_chain );
     }
 
     /* Initialize the dimensions of the video window */
     InitWindowSize( p_vout, &p_vout->i_window_width,
                     &p_vout->i_window_height );
 
-
+    /* Create the vout thread */
     p_vout->p_module = module_Need( p_vout,
                            ( p_vout->psz_filter_chain &&
                                *p_vout->psz_filter_chain ) ?
@@ -357,15 +395,62 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
         return NULL;
     }
 
-    /* Create thread and set locks */
-    vlc_mutex_init( p_vout, &p_vout->picture_lock );
-    vlc_mutex_init( p_vout, &p_vout->subpicture_lock );
-    vlc_mutex_init( p_vout, &p_vout->change_lock );
+    p_vout->p_text_renderer_module = module_Need( p_vout, "text renderer", 
+						  NULL );
+    if( p_vout->p_text_renderer_module == NULL )
+    {
+	msg_Warn( p_vout, "no suitable text renderer module" );
+	p_vout->pf_add_string = NULL;
+    }
 
-    vlc_object_attach( p_vout, p_parent );
+    /* Create a few object variables for interface interaction */
+    var_Create( p_vout, "fullscreen", VLC_VAR_BOOL );
+    text.psz_string = _("Fullscreen");
+    var_Change( p_vout, "fullscreen", VLC_VAR_SETTEXT, &text, NULL );
+    var_Change( p_vout, "fullscreen", VLC_VAR_INHERITVALUE, &val, NULL );
+    if( val.b_bool )
+    {
+        /* user requested fullscreen */
+        p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
+    }
+    var_AddCallback( p_vout, "fullscreen", FullscreenCallback, NULL );
+
+    var_Create( p_vout, "deinterlace", VLC_VAR_STRING | VLC_VAR_HASCHOICE );
+    text.psz_string = _("Deinterlace");
+    var_Change( p_vout, "deinterlace", VLC_VAR_SETTEXT, &text, NULL );
+    val.psz_string = ""; text.psz_string = _("Disable");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "discard"; text.psz_string = _("Discard");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "blend"; text.psz_string = _("Blend");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "mean"; text.psz_string = _("Mean");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "bob"; text.psz_string = _("Bob");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "linear"; text.psz_string = _("Linear");
+    var_Change( p_vout, "deinterlace", VLC_VAR_ADDCHOICE, &val, &text );
+    if( var_Get( p_vout, "deinterlace-mode", &val ) == VLC_SUCCESS )
+    {
+        var_Set( p_vout, "deinterlace", val );
+    }
+    var_AddCallback( p_vout, "deinterlace", DeinterlaceCallback, NULL );
+
+    /* Calculate delay created by internal caching */
+    p_input_thread = (input_thread_t *)vlc_object_find( p_vout,
+                                           VLC_OBJECT_INPUT, FIND_ANYWHERE );
+    if( p_input_thread )
+    {
+        p_vout->i_pts_delay = p_input_thread->i_pts_delay;
+        vlc_object_release( p_input_thread );
+    }
+    else
+    {
+        p_vout->i_pts_delay = DEFAULT_PTS_DELAY;
+    }
 
     if( vlc_thread_create( p_vout, "video output", RunThread,
-                           VLC_THREAD_PRIORITY_OUTPUT, VLC_FALSE ) )
+                           VLC_THREAD_PRIORITY_OUTPUT, VLC_TRUE ) )
     {
         msg_Err( p_vout, "out of memory" );
         module_Unneed( p_vout, p_vout->p_module );
@@ -385,15 +470,38 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
  * update using one of the THREAD_* constants.
  *****************************************************************************/
 void vout_Destroy( vout_thread_t *p_vout )
-{
+{ 
+    vlc_object_t *p_playlist;
+    
     /* Request thread destruction */
     p_vout->b_die = VLC_TRUE;
     vlc_thread_join( p_vout );
 
     var_Destroy( p_vout, "intf-change" );
 
+    p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST, 
+                                  FIND_ANYWHERE );
+
     /* Free structure */
     vlc_object_destroy( p_vout );
+
+    /* If it was the last vout, tell the interface to show up */
+    if( p_playlist != NULL )
+    {
+        vout_thread_t *p_another_vout = vlc_object_find( p_playlist, 
+                                            VLC_OBJECT_VOUT, FIND_ANYWHERE );
+        if( p_another_vout == NULL )
+        {
+            vlc_value_t val;
+            val.b_bool = VLC_TRUE;
+            var_Set( p_playlist, "intf-show", val );
+        }
+        else
+        {
+            vlc_object_release( p_another_vout );
+        }
+        vlc_object_release( p_playlist );
+    }
 }
 
 /*****************************************************************************
@@ -591,6 +699,10 @@ static void RunThread( vout_thread_t *p_vout)
      * Initialize thread
      */
     p_vout->b_error = InitThread( p_vout );
+
+    /* signal the creation of the vout */
+    vlc_thread_ready( p_vout );
+
     if( p_vout->b_error )
     {
         /* Destroy thread structures allocated by Create and InitThread */
@@ -609,7 +721,7 @@ static void RunThread( vout_thread_t *p_vout)
         display_date = 0;
         current_date = mdate();
 
-#ifdef STATS
+#if 0
         p_vout->c_loops++;
         if( !(p_vout->c_loops % VOUT_STATS_NB_LOOPS) )
         {
@@ -651,7 +763,7 @@ static void RunThread( vout_thread_t *p_vout)
                     }
                 }
             }
-    
+
             /* If we found better than the last picture, destroy it */
             if( p_last_picture && p_picture != p_last_picture )
             {
@@ -698,9 +810,9 @@ static void RunThread( vout_thread_t *p_vout)
 
                 continue;
             }
-#if 0
-            /* Removed because it causes problems for some people --Meuuh */
-            if( display_date > current_date + VOUT_BOGUS_DELAY )
+
+            if( display_date >
+                current_date + p_vout->i_pts_delay +  VOUT_BOGUS_DELAY )
             {
                 /* Picture is waaay too early: it will be destroyed */
                 vlc_mutex_lock( &p_vout->picture_lock );
@@ -716,13 +828,14 @@ static void RunThread( vout_thread_t *p_vout)
                     p_picture->i_status = DESTROYED_PICTURE;
                     p_vout->i_heap_size--;
                 }
-                intf_WarnMsg( 1, "vout warning: early picture skipped "
-                              "("I64Fd")", display_date - current_date );
+                msg_Warn( p_vout, "vout warning: early picture skipped "
+                          "("I64Fd")", display_date - current_date
+                          - p_vout->i_pts_delay );
                 vlc_mutex_unlock( &p_vout->picture_lock );
 
                 continue;
             }
-#endif
+
             if( display_date > current_date + VOUT_DISPLAY_DELAY )
             {
                 /* A picture is ready to be rendered, but its rendering date
@@ -768,7 +881,7 @@ static void RunThread( vout_thread_t *p_vout)
         /*
          * Call the plugin-specific rendering method if there is one
          */
-        if( p_picture != NULL && p_vout->pf_render )
+        if( p_picture != NULL && p_directbuffer != NULL && p_vout->pf_render )
         {
             /* Render the direct buffer returned by vout_RenderPicture */
             p_vout->pf_render( p_vout, p_directbuffer );
@@ -777,7 +890,7 @@ static void RunThread( vout_thread_t *p_vout)
         /*
          * Sleep, wake up
          */
-        if( display_date != 0 )
+        if( display_date != 0 && p_directbuffer != NULL )
         {
             /* Store render time using a sliding mean */
             p_vout->render_time += mdate() - current_date;
@@ -804,7 +917,7 @@ static void RunThread( vout_thread_t *p_vout)
         /*
          * Display the previously rendered picture
          */
-        if( p_picture != NULL )
+        if( p_picture != NULL && p_directbuffer != NULL )
         {
             /* Display the direct buffer returned by vout_RenderPicture */
             if( p_vout->pf_display )
@@ -855,7 +968,8 @@ static void RunThread( vout_thread_t *p_vout)
             }
 
             /* Need to reinitialise the chroma plugin */
-            p_vout->chroma.p_module->pf_deactivate( VLC_OBJECT(p_vout) );
+            if( p_vout->chroma.p_module->pf_deactivate )
+                p_vout->chroma.p_module->pf_deactivate( VLC_OBJECT(p_vout) );
             p_vout->chroma.p_module->pf_activate( VLC_OBJECT(p_vout) );
         }
     }
@@ -916,7 +1030,7 @@ static void EndThread( vout_thread_t *p_vout )
     {
         module_Unneed( p_vout, p_vout->chroma.p_module );
     }
-
+    
     /* Destroy all remaining pictures */
     for( i_index = 0; i_index < 2 * VOUT_MAX_PICTURES; i_index++ )
     {
@@ -936,6 +1050,9 @@ static void EndThread( vout_thread_t *p_vout )
         }
     }
 
+    if( p_vout->p_text_renderer_module )
+        module_Unneed( p_vout, p_vout->p_text_renderer_module );
+    
     /* Destroy translation tables */
     p_vout->pf_end( p_vout );
 
@@ -1055,29 +1172,33 @@ static void MaskToShift( int *pi_left, int *pi_right, uint32_t i_mask )
 static void InitWindowSize( vout_thread_t *p_vout, int *pi_width,
                             int *pi_height )
 {
+    vlc_value_t val;
     int i_width, i_height;
     uint64_t ll_zoom;
 
 #define FP_FACTOR 1000                             /* our fixed point factor */
 
-    i_width = config_GetInt( p_vout, "width" );
-    i_height = config_GetInt( p_vout, "height" );
-    ll_zoom = (uint64_t)( FP_FACTOR * config_GetFloat( p_vout, "zoom" ) );
+    var_Get( p_vout, "width", &val );
+    i_width = val.i_int;
+    var_Get( p_vout, "height", &val );
+    i_height = val.i_int;
+    var_Get( p_vout, "zoom", &val );
+    ll_zoom = (uint64_t)( FP_FACTOR * val.f_float );
 
-    if( (i_width >= 0) && (i_height >= 0))
+    if( i_width > 0 && i_height > 0)
     {
         *pi_width = (int)( i_width * ll_zoom / FP_FACTOR );
         *pi_height = (int)( i_height * ll_zoom / FP_FACTOR );
         return;
     }
-    else if( i_width >= 0 )
+    else if( i_width > 0 )
     {
         *pi_width = (int)( i_width * ll_zoom / FP_FACTOR );
         *pi_height = (int)( i_width * ll_zoom * VOUT_ASPECT_FACTOR /
                             p_vout->render.i_aspect / FP_FACTOR );
         return;
     }
-    else if( i_height >= 0 )
+    else if( i_height > 0 )
     {
         *pi_height = (int)( i_height * ll_zoom / FP_FACTOR );
         *pi_width = (int)( i_height * ll_zoom * p_vout->render.i_aspect /
@@ -1111,7 +1232,111 @@ int vout_VarCallback( vlc_object_t * p_this, const char * psz_variable,
 {
     vout_thread_t * p_vout = (vout_thread_t *)p_this;
     vlc_value_t val;
-    val.b_bool = 1;
+    val.b_bool = VLC_TRUE;
     var_Set( p_vout, "intf-change", val );
-    return 0;
+    return VLC_SUCCESS;
+}
+
+/*****************************************************************************
+ * object variables callbacks: a bunch of object variables are used by the
+ * interfaces to interact with the vout.
+ *****************************************************************************/
+static int FullscreenCallback( vlc_object_t *p_this, char const *psz_cmd,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    input_thread_t *p_input;
+    vlc_value_t val;
+
+    p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
+
+    p_input = (input_thread_t *)vlc_object_find( p_this, VLC_OBJECT_INPUT,
+                                                 FIND_PARENT );
+    if( p_input )
+    {
+        /* Modify input as well because the vout might have to be restarted */
+        var_Create( p_input, "fullscreen", VLC_VAR_BOOL );
+        var_Set( p_input, "fullscreen", newval );
+
+        vlc_object_release( p_input );
+    }
+
+    val.b_bool = VLC_TRUE;
+    var_Set( p_vout, "intf-change", val );
+    return VLC_SUCCESS;
+}
+
+static int DeinterlaceCallback( vlc_object_t *p_this, char const *psz_cmd,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    input_thread_t *p_input;
+    vlc_value_t val;
+
+    char *psz_mode = newval.psz_string;
+    char *psz_filter;
+    unsigned int  i;
+
+    psz_filter = config_GetPsz( p_vout, "filter" );
+
+    if( !psz_mode || !*psz_mode )
+    {
+        config_PutPsz( p_vout, "filter", "" );
+    }
+    else
+    {
+        if( !psz_filter || !*psz_filter )
+        {
+            config_PutPsz( p_vout, "filter", "deinterlace" );
+        }
+        else
+        {
+            if( strstr( psz_filter, "deinterlace" ) == NULL )
+            {
+                psz_filter = realloc( psz_filter, strlen( psz_filter ) + 20 );
+                strcat( psz_filter, ",deinterlace" );
+            }
+            config_PutPsz( p_vout, "filter", psz_filter );
+        }
+    }
+
+    if( psz_filter ) free( psz_filter );
+
+
+    p_input = (input_thread_t *)vlc_object_find( p_this, VLC_OBJECT_INPUT,
+                                                 FIND_PARENT );
+    if( !p_input ) return VLC_EGENERIC;
+
+    if( psz_mode && *psz_mode )
+    {
+        val.psz_string = psz_mode;
+        var_Set( p_vout, "deinterlace-mode", val );
+        /* Modify input as well because the vout might have to be restarted */
+        var_Create( p_input, "deinterlace-mode", VLC_VAR_STRING );
+        var_Set( p_input, "deinterlace-mode", val );
+    }
+
+    /* now restart all video streams */
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
+    p_vout->b_filter_change = VLC_TRUE;
+
+#define ES p_input->stream.pp_es[i]
+
+    for( i = 0 ; i < p_input->stream.i_es_number ; i++ )
+    {
+        if( ( ES->i_cat == VIDEO_ES ) && ES->p_decoder_fifo != NULL )
+        {
+            input_UnselectES( p_input, ES );
+            input_SelectES( p_input, ES );
+        }
+#undef ES
+    }
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
+    vlc_object_release( p_input );
+
+    val.b_bool = VLC_TRUE;
+    var_Set( p_vout, "intf-change", val );
+    return VLC_SUCCESS;
 }
